---------------------------------------------------------------------------------------------------
-- ConfigWindow Functions
---------------------------------------------------------------------------------------------------
NNConfigForm = {}

function NNConfigForm:SetCheck(wndParent, strChild, bValue)
	wndTemp = wndParent:FindChild(strChild)
	if not bValue then
		wndTemp:SetCheck(false)
		return false
	else
		wndTemp:SetCheck(true)
		return true
	end
end

function NNConfigForm:GetCheck(wndParent, strCheck)
	local wndTest = wndParent:FindChild(strCheck)
	if wndTest:IsChecked() then
		return true
	end
	return nil
end


function NNConfigForm:Load()
	self.xmlDoc = XmlDoc.CreateFromFile("ConfigForm.xml")
	if not self.xmlDoc then
		Apollo.AddAddonErrorText("Could not load ConfigForm")
	end	
end

function NNConfigForm:Init()
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "ConfigWindow", nil, self)
	self.wndContent = self.wndMain:FindChild("Content")
	
	self.wndWindow = Apollo.LoadForm(self.xmlDoc, "ContentWindow", self.wndContent, self)
	self.wndAuto = Apollo.LoadForm(self.xmlDoc, "ContentAuto", self.wndContent, self)
	self.wndShared = Apollo.LoadForm(self.xmlDoc, "ContentShared", self.wndContent, self)

	self.wndMain:FindChild("BtnWindows"):SetCheck(true)
	self.wndWindow:Show(true)
	self.wndAuto:Show(false)
	self.wndShared:Show(false)
end

function NNConfigForm:Open()
	if self.wndMain and self.wndMain:IsShown() then
		self:Close()
		return
	elseif not self.wndMain or not self.wndMain:IsValid() then
		self:Init()
	end
	local tCharacter = NeighborNotes.tCharacter
	local tRealm = NeighborNotes.tRealm
	
	self.wndWindow:FindChild("CheckboxOpenWithSocial"):SetCheck(tCharacter.bOpenWithSocial)
	self.wndWindow:FindChild("CheckboxCloseWithSocial"):SetCheck(tCharacter.bCloseWithSocial)
	self.wndWindow:FindChild("CheckboxOpenMinimum"):SetCheck(tRealm.bOpenMinimized)
	self.wndWindow:FindChild("CheckboxCloseOnExit"):SetCheck(tRealm.bCloseOnExit)
	self.wndWindow:FindChild("CheckboxVisitNext"):SetCheck(tCharacter.bOnVisitSelectNext)
	self.wndWindow:FindChild("CheckboxStoreNonNeighbors"):SetCheck(tCharacter.bGatherNonNeighbors)

	self.wndShared:FindChild("CheckboxShowPlugs"):SetCheck(tCharacter.bShowDataInChat)
	self.wndShared:FindChild("CheckboxShowVisits"):SetCheck(tCharacter.bShowChatVisits)
	self.wndShared:FindChild("CheckboxShowHarvests"):SetCheck(tCharacter.bShowChatHarvests)
	self.wndShared:FindChild("CheckboxShareData"):SetCheck(tCharacter.bShareData)
	if tCharacter.bShareData then
		self.wndShared:FindChild("CheckboxShareHarvest"):SetCheck(tCharacter.bShareHarvest)
		self.wndShared:FindChild("CheckboxShowInSearch"):SetCheck(not tCharacter.bHideFromSearch)
		self.wndShared:FindChild("CheckboxShareHarvest"):Enable(true)
		self.wndShared:FindChild("CheckboxShowInSearch"):Enable(true)
	else
		self.wndShared:FindChild("CheckboxShareHarvest"):Enable(false)
		self.wndShared:FindChild("CheckboxShowInSearch"):Enable(false)
	end
	self:SetCheck(self.wndShared, "CheckboxDisableICC", tCharacter.bDisableICC)
	if not tCharacter.tAutomation then
		tCharacter.tAutomation = {}
	end
	if self:SetCheck(self.wndAuto, "cbAAAccept", tCharacter.tAutomation.bAA) then
		if self:SetCheck(self.wndAuto, "cbAAReject", tCharacter.tAutomation.bAAReject) then
			self:OnAAReject()
		elseif self:SetCheck(self.wndAuto, "cbAAHarvestable", tCharacter.tAutomation.bAAHarvest) then
			self:OnAAHarvestable()
			local strSharePercent = "50"
			if tCharacter.tAutomation.eAAHarvestShare then
				if tCharacter.tAutomation.eAAHarvestShare == NNNetwork.eShare["0"] then
					strSharePercent = "0"
				elseif tCharacter.tAutomation.eAAHarvestShare == NNNetwork.eShare["25"] then
					strSharePercent = "25"
				elseif tCharacter.tAutomation.eAAHarvestShare == NNNetwork.eShare["50"] then
					strSharePercent = "50"
				elseif tCharacter.tAutomation.eAAHarvestShare == NNNetwork.eShare["75"] then
					strSharePercent = "75"
				elseif tCharacter.tAutomation.eAAHarvestShare == NNNetwork.eShare["100"] then
					strSharePercent = "100"
				end
			end
			self.wndAuto:FindChild("cbSharePercent"):SetText(strSharePercent)
			self:SetCheck(self.wndAuto, "cbAAHarvestSkill", tCharacter.tAutomation.bAAHarvestSkill)
		end
		self:SetCheck(self.wndAuto, "cbAAGuild", tCharacter.tAutomation.bAAGuild)
		self:SetCheck(self.wndAuto, "cbAACircle", tCharacter.tAutomation.bAACircle)
		self:SetCheck(self.wndAuto, "cbAAMyHarvest", tCharacter.tAutomation.bAAMyHarvest)
	end
	self:SetCheck(self.wndAuto, "cbAIFriend", tCharacter.tAutomation.bAIFriend)
	self:SetCheck(self.wndAuto, "cbAIGuild", tCharacter.tAutomation.bAIGuild)
	self:SetCheck(self.wndAuto, "cbAICircle", tCharacter.tAutomation.bAICircle)
	if self:SetCheck(self.wndAuto, "cbARemove", tCharacter.tAutomation.bARemove) then
		self:OnAutoRemove()
		if not tCharacter.tAutomation.nARemoveTime then
			self.wndAuto:FindChild("cbARemTime"):SetText("3")
		else
			self.wndAuto:FindChild("cbARemTime"):SetText(tostring(tCharacter.tAutomation.nARemoveTime))
		end
		self:SetCheck(self.wndAuto, "cbARemGuild", tCharacter.tAutomation.bARemoveGuild)
		self:SetCheck(self.wndAuto, "cbARemCircle", tCharacter.tAutomation.bARemoveCircle)
	end
	self:SetCheck(self.wndAuto, "cbARepHarvest", tCharacter.tAutomation.bARepairHarvest)
	self:SetCheck(self.wndAuto, "cbARepOther", tCharacter.tAutomation.bARepairOther)
	self:SetCheck(self.wndAuto, "cbARepNotifyHarvest", tCharacter.tAutomation.bANotifyHarvest)
	self:SetCheck(self.wndAuto, "cbARepNotifyHarvestAlt", tCharacter.tAutomation.bANotifyHarvestAlt)
	
	self:OnAutoAccept()
	self:OnAAReject()
	self:OnAutoRemove()
	
	self.wndMain:Show(true)
	self.wndMain:ToFront()
end

function NNConfigForm:Close()
	if self.wndMain and self.wndMain:IsShown() then
		self.wndMain:Close()
	end
end

function NNConfigForm:OnNavBtn( wndHandler, wndControl, eMouseButton )
	self.wndWindow:Show(false)
	self.wndAuto:Show(false)
	self.wndShared:Show(false)
	if wndControl:GetName() == "BtnWindows" then
		self.wndWindow:Show(true)
	elseif wndControl:GetName() == "BtnList" then
		self.wndAuto:Show(true)
	elseif wndControl:GetName() == "BtnShared" then
		self.wndShared:Show(true)
	end
end

function NNConfigForm:OnOKBtn( wndHandler, wndControl, eMouseButton)
	local tCharacter = NeighborNotes.tCharacter
	local tRealm = NeighborNotes.tRealm
	tCharacter.bOpenWithSocial = self:GetCheck(self.wndWindow, "CheckboxOpenWithSocial")
	tCharacter.bCloseWithSocial = self:GetCheck(self.wndWindow, "CheckboxCloseWithSocial")

	tRealm.bOpenMinimized = self:GetCheck(self.wndWindow, "CheckboxOpenMinimum")
	tRealm.bCloseOnExit = self:GetCheck(self.wndWindow, "CheckboxCloseOnExit")

	tCharacter.bOnVisitSelectNext = self:GetCheck(self.wndWindow, "CheckboxVisitNext")
	tCharacter.bGatherNonNeighbors = self:GetCheck(self.wndWindow, "CheckboxStoreNonNeighbors")
	tCharacter.bShowDataInChat = self:GetCheck(self.wndShared, "CheckboxShowPlugs")
	tCharacter.bShowChatVisits = self:GetCheck(self.wndShared, "CheckboxShowVisits")
	tCharacter.bShowChatHarvests = self:GetCheck(self.wndShared, "CheckboxShowHarvests")
	tCharacter.bShareData = self:GetCheck(self.wndShared, "CheckboxShareData")
	tCharacter.bDisableICC = self:GetCheck(self.wndShared, "CheckboxDisableICC")
	if not tCharacter.bShareData then
		tCharacter.bShareHarvest = nil
		tCharacter.bHideFromSearch = nil
	else
		tCharacter.bShareHarvest = self:GetCheck(self.wndShared, "CheckboxShareHarvest")
		tCharacter.bHideFromSearch = not self.wndShared:FindChild("CheckboxShowInSearch"):IsChecked()
	end
	--Automation
	tCharacter.tAutomation.bAA = self:GetCheck(self.wndAuto, "cbAAAccept")
	if tCharacter.tAutomation.bAA then
		tCharacter.tAutomation.bAAGuild = self:GetCheck(self.wndAuto, "cbAAGuild")
		tCharacter.tAutomation.bAACircle = self:GetCheck(self.wndAuto, "cbAACircle")
		tCharacter.tAutomation.bAAReject = self:GetCheck(self.wndAuto, "cbAAReject")
		if not tCharacter.tAutomation.bAAReject then
			tCharacter.tAutomation.bAAHarvest = self:GetCheck(self.wndAuto, "cbAAHarvestable")
			tCharacter.tAutomation.bAAMyHarvest = self:GetCheck(self.wndAuto, "cbAAMyHarvest")			
			if tCharacter.tAutomation.bAAHarvest then
				tCharacter.tAutomation.bAAHarvestSkill = self:GetCheck(self.wndAuto, "cbAAHarvestSkill")
				tCharacter.tAutomation.eAAHarvestShare = NNNetwork.eShare[self.wndAuto:FindChild("cbSharePercent"):GetText()]
			end

		end
	end
	tCharacter.tAutomation.bAIFriend = self:GetCheck(self.wndAuto, "cbAIFriend")
	tCharacter.tAutomation.bAIGuild = self:GetCheck(self.wndAuto, "cbAIGuild")
	tCharacter.tAutomation.bAICircle = self:GetCheck(self.wndAuto, "cbAICircle")
	tCharacter.tAutomation.bARemove = self:GetCheck(self.wndAuto, "cbARemove")
	if tCharacter.tAutomation.bARemove then
		tCharacter.tAutomation.bARemoveGuild = self:GetCheck(self.wndAuto, "cbARemGuild")
		tCharacter.tAutomation.bARemoveCircle = self:GetCheck(self.wndAuto, "cbARemCircle")
		tCharacter.tAutomation.nARemoveTime = tonumber(self.wndAuto:FindChild("cbARemTime"):GetText())
	end
	tCharacter.tAutomation.bARepairHarvest = self:GetCheck(self.wndAuto, "cbARepHarvest")
	tCharacter.tAutomation.bARepairOther = self:GetCheck(self.wndAuto, "cbARepOther")
	tCharacter.tAutomation.bANotifyHarvest = self:GetCheck(self.wndAuto, "cbARepNotifyHarvest")
	tCharacter.tAutomation.bANotifyHarvestAlt = self:GetCheck(self.wndAuto, "cbARepNotifyHarvestAlt")
	
	NeighborNotes:SetInviteAddon()
	NeighborNotes:AutoRepair()
	NeighborNotes:AutoRemove()
	NeighborNotes:RepairNotify()
	NeighborNotes:RefreshList()
	self:Close()	
end

function NNConfigForm:OnCloseBtn( wndHandler, wndControl, eMouseButton )
	self:Close()
end

---------------------------------------------------------------------------------------------------
-- ContentShared Functions
---------------------------------------------------------------------------------------------------
function NNConfigForm:OnShareDataCheck( wndHandler, wndControl, eMouseButton )
	self.wndShared:FindChild("CheckboxShareHarvest"):Enable(wndControl:IsChecked())
	self.wndShared:FindChild("CheckboxShowInSearch"):Enable(wndControl:IsChecked())
end

function NNConfigForm:OnAutoAccept( wndHandler, wndControl, eMouseButton )
	if not wndControl then
		wndControl = self.wndAuto:FindChild("cbAAAccept")
	end
	if not wndControl:IsChecked() then
		self.wndAuto:FindChild("cbAAHarvestable"):SetCheck(false)
	end
	self.wndAuto:FindChild("cbAAHarvestable"):Enable(wndControl:IsChecked())
	self.wndAuto:FindChild("cbSharePercent"):Enable(wndControl:IsChecked())
	self:OnAAHarvestable()
	self.wndAuto:FindChild("cbAAReject"):Enable(wndControl:IsChecked())
	self.wndAuto:FindChild("cbAAMyHarvest"):Enable(wndControl:IsChecked())
	self.wndAuto:FindChild("cbAAGuild"):Enable(wndControl:IsChecked())
	self.wndAuto:FindChild("cbAACircle"):Enable(wndControl:IsChecked())
end

function NNConfigForm:OnAAHarvestable( wndHandler, wndControl, eMouseButton )
	if not wndControl then
		wndControl = self.wndAuto:FindChild("cbAAHarvestable")
	end
	self.wndAuto:FindChild("cbAAHarvestSkill"):Enable(wndControl:IsChecked())
	self.wndAuto:FindChild("cbSharePercent"):Enable(wndControl:IsChecked())
end

function NNConfigForm:OnAAReject( wndHandler, wndControl, eMouseButton )
	if not wndControl then
		wndControl = self.wndAuto:FindChild("cbAAReject")
	end
	self.wndAuto:FindChild("cbAAMyHarvest"):Enable(not wndControl:IsChecked())
	self.wndAuto:FindChild("cbSharePercent"):Enable(not wndControl:IsChecked())
	self.wndAuto:FindChild("cbAAHarvestSkill"):Enable(not wndControl:IsChecked())
	if wndControl:IsChecked() then
		self.wndAuto:FindChild("cbAAHarvestable"):Enable(false)
		self.wndAuto:FindChild("cbSharePercent"):Enable(false)
	else
		self.wndAuto:FindChild("cbAAHarvestable"):Enable(true)
		self:OnAAHarvestable()
	end
	
end

function NNConfigForm:OnResetBtn( wndHandler, wndControl, eMouseButton )
	wndControl:FindChild("ResetConfirm"):Show(wndControl:IsChecked())
end

function NNConfigForm:OnResetConfirmBtn( wndHandler, wndControl, eMouseButton )
	NeighborNotes:Reset()
end


function NNConfigForm:OnAutoRemove( wndHandler, wndControl, eMouseButton )
	if not wndControl then
		wndControl = self.wndAuto:FindChild("cbARemove")
	end
	self.wndAuto:FindChild("cbARemTime"):Enable(wndControl:IsChecked())
	self.wndAuto:FindChild("cbARemGuild"):Enable(wndControl:IsChecked())
	self.wndAuto:FindChild("cbARemCircle"):Enable(wndControl:IsChecked())
end

function NNConfigForm:OnComboBox( wndHandler, wndControl, eMouseButton )
	wndControl:FindChild("ComboBoxList"):Show(wndControl:IsChecked())
	if wndControl:IsChecked() then
		wndControl:FindChild("ComboBoxList"):BringToFront()
	end
end

function NNConfigForm:OnComboBoxItem( wndHandler, wndControl, eMouseButton )
	wndControl:GetParent():GetParent():SetText(wndControl:GetText())
	wndControl:GetParent():GetParent():SetCheck(false)	
	wndControl:GetParent():Show(false)
end


