-----------------------------------------------------------------------------------------------
-- Client Lua Script for NeighborNotes
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
function string.IsEmpty(strString)
	if strString == nil or strString == "" then
		return true
	end
	return false
end

function table.IsEmpty(tTable)
	if tTable ~= nil then
		if #tTable > 0 then
			return false
		end
		for unknown, unknown2 in pairs(tTable) do
			if unknown then
				return false
			end
		end
	end
	return true
end

require "Window"
require "Unit"
require "string"
require "CraftingLib"
require "FriendshipLib"
require "GuildLib"
require "HousingLib"

-----------------------------------------------------------------------------------------------
-- NeighborNotes Module Definition
-----------------------------------------------------------------------------------------------
NeighborNotes = {}
NeighborNotes.nVersion = 255

NeighborNotes.tCharacter = { }
NeighborNotes.tRealm = { 
	["nCharacterCount"] = 0,}
NeighborNotes.tAccount = { }

NeighborNotes.tEnums = {}
NeighborNotes.tLocale = {}
NeighborNotes.tWindowData = {}
NeighborNotes.tWindowData.tMain = {}
NeighborNotes.tWindowData.tSmall = {}
NeighborNotes.fnSort = nil
NeighborNotes.fnLastSort = nil

NeighborNotes.tHarvestCache = {}
NeighborNotes.tHarvestCache.nToMe = 0
NeighborNotes.tHarvestCache.nToOwner = 0

NeighborNotes.tIcons = {
	["strMining"] = "IconSprites:Icon_CraftingUI_Item_Crafting_Powder_AllSpice",
	["strRelic"] = "CRB_GuildSprites:sprGuild_TribalMask",
	["strTree"] = "IconSprites:Icon_MapNode_Map_Node_Tree",
}

NeighborNotes.tLiveData = {
	["bShowSplash"] = true,
	["strNeighborName"] = "",
	["tSelectedNeighbor"] = nil,
	["tActivePlayerSkills"] = {},
	["tQueuedUnits"] = {},
	["tPlugs"]		= {},	-- tPlugs = { [nPlugID] = strNonLocalizedPlugName, }
	["tPlugUnits"]	= {},	-- tUnits = { [strLocalizedUnitName] = nPlugID, }
	["tPlugNames"] = {},	-- tPlugNames = { [strNonLocalizedPlugName] = nPlugID }
	["tGroups"] 	= {},	-- [GroupName] = { nType, tRoster = {} }
	["nScan"] = 0,
	["tInvites"] = {},
	["tSearch"] = {},
	["tFilters"] = {
		["bShowSearch"] = false,
		["bShowNeighbors"] = true,
		["bShowNonNeighbors"] = false,
		["nPlug"] = nil,
	},
}

NeighborNotes.tColors = {
	crOnline 	= ApolloColor.new("UI_TextHoloBodyHighlight"),
	crOffline 	= ApolloColor.new("UI_BtnTextGrayNormal"),
	crNeutral 	= ApolloColor.new("gray"),
}

-----------------------------------------------------------------------------------------------
-- Generic Addon Functions
-----------------------------------------------------------------------------------------------

-- This is the constructor
function NeighborNotes:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

-- Registers our Addon with the Apollo UI
function NeighborNotes:Init()
	local bHasConfiguration = true
	local strConfigText = "Neighbor Notes"
	local tDependencies = { }

    Apollo.RegisterAddon(self, bHasConfiguration, strConfigText, tDependencies)
end

-- Loads the initial variables and registers the NN Callback function (for when the window is loaded)
function NeighborNotes:OnLoad()
	self.tCharacter = NeighborNotes:CharacterDefaults()
	self.tRealm = NeighborNotes:RealmDefaults()
	
	Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)

	-- Create the forms
	NNConfigForm:Load()
	NNSmallForm:Load()
	NNMainForm:Load()

	-- Register event handlers
	Apollo.RegisterEventHandler("ToggleAddon_NN", 				"OpenNN", self)
	Apollo.RegisterEventHandler("HousingNeighborUpdate",		"RefreshList", self)
	Apollo.RegisterEventHandler("HousingNeighborsLoaded", 		"RefreshList", self)
	Apollo.RegisterEventHandler("OnFriendshipUpdate",			"OnFriendshipUpdate", self)
	Apollo.RegisterEventHandler("OnFriendshipUpdateOnline",		"OnFriendshipUpdate", self)
	Apollo.RegisterEventHandler("OnFriendshipAccountDataUpdate","OnFriendshipUpdate", self)
	Apollo.RegisterEventHandler("GuildRoster",					"OnGuildRosterReceived", self)
	Apollo.RegisterEventHandler("GuildMemberChange",			"OnGuildMemberChange", self)
	Apollo.RegisterEventHandler("VarChange_ZoneName",			"OnChangeZoneName", self)
	Apollo.RegisterEventHandler("HarvestItemsSentToOwner", "OnHarvest", self)
	Apollo.RegisterEventHandler("LootedItem", "OnLoot", self)

	-- Monitor the social window events		
	Apollo.RegisterEventHandler("EventGeneric_OpenSocialPanel", "OnSocialWindowToggle", self)
	Apollo.RegisterEventHandler("ToggleSocialWindow", 			"OnSocialWindowToggle", self)
	Apollo.RegisterEventHandler("SocialWindowHasBeenClosed",	"OnSocialWindowClose", self)
	Apollo.RegisterEventHandler("GenericEvent_InitializeNeighbors", "OnSocialWindowToggle", self)
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", 	"OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("PlayerCurrencyChanged", "OnPlayerCurrencyChanged", self)
	
	-- Register Slash Commands				
	Apollo.RegisterSlashCommand("neighbornotes", 				"OpenNN", self)
	Apollo.RegisterSlashCommand("nn", 							"OpenNN", self)
	Apollo.RegisterSlashCommand("nnv",	 						"OnNNVisit", self)
	Apollo.RegisterSlashCommand("nnscan",						"OnStartScan", self)
	Apollo.RegisterSlashCommand("home",							"OnTeleportHomeBtn", NNMainForm)

	self:SetLocaleData()

	end

----------------------------------------------------------------------------------------------
-- Global Event Handlers
----------------------------------------------------------------------------------------------

-- Saves data when the game is closed, or when the UI is reloaded
function NeighborNotes:OnSave(eType)
	local tSave = {}
	-- Account Level
	if eType == GameLib.CodeEnumAddonSaveLevel.Account then
		tSave = self.tAccount
	end
	-- Realm Level
	if eType == GameLib.CodeEnumAddonSaveLevel.Realm then
		tSave = self.tRealm
	end
	-- Character Level
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		tSave = self.tCharacter
	end
	return tSave
end

-- Loads data when the game is closed, or when the UI is reloaded
function NeighborNotes:OnRestore(eType, tLoad)
	if eType == GameLib.CodeEnumAddonSaveLevel.Account then
		if not tLoad then
			tLoad.bShowSplash = true
		elseif not tLoad.nVersion or tLoad.nVersion ~= self.nVersion then
				tLoad.bShowSplash = true
		end
		self.tAccount = tLoad
		self.tAccount.nVersion = self.nVersion
	elseif eType == GameLib.CodeEnumAddonSaveLevel.Realm then
		if not tLoad then
			tLoad = self:RealmDefaults()
		else
			if not tLoad.nVersion or tLoad.nVersion ~= self.nVersion then
				if not tLoad.nVersion or tLoad.nVersion < 252 then
					if tLoad.tNeighborList ~= nil then
						for strNeighborName, tData in pairs(tLoad.tNeighborList) do
							if tData.nNodeID > 0 and tData.nNodeID < 30 then -- check for old NodeIDs
								tData.nNodeID = tData.nNodeID + 50
							end
						end
					end
				end
			end
			if	tLoad.tNeighborList ~= nil then
				for strNeighborName, tData in pairs(tLoad.tNeighborList) do
					if tData.tPlugs then
						if type(tData.tPlugs) ~= "table" then
							tData.tPlugs = {}
						end
					end
					if table.IsEmpty(tLoad.tHarvestHistory) then
						tData.tHarvestHistory = nil
					end
					if string.IsEmpty(tData.strNote) then
						tData.strNote = nil
					end
					if not tData.nNodeID then
						tData.nNodeID = 0
					end
				end
			else
				tLoad.tNeighborList = {}
			end
		end
		self.tRealm = tLoad
		self.tRealm.nVersion = self.nVersion
	elseif eType == GameLib.CodeEnumAddonSaveLevel.Character then
		if not tLoad then
			tLoad = self:CharacterDefaults()
		else
			if not tLoad.nVersion or tLoad.nVersion ~= self.nVersion then
				if not tLoad.nVersion or tLoad.nVersion < 250 then
					tLoad.tAutomation = self:CharacterDefaults().tAutomation
					if tLoad.tFilters ~= nil then
						tLoad.tFilters.strMiscOptions = nil
						tLoad.tFilters.bShowNotes = true
					end
				end
				if not tLoad.nVersion or tLoad.nVersion < 251 then
					if tLoad.tFilters ~= nil then
						tLoad.tFilters.bShowAllCharacters = nil	-- No longer used
						tLoad.tFilters.bShowInSearch = nil		-- No longer used
					end
				end
			end
			if not tLoad.tAutomation then
				tLoad.tAutomation = self:CharacterDefaults().tAutomation
			end 
			if not tLoad.tFilters then
				tLoad.tFilters = self:CharacterDefaults().tFilters
			end
		end
		self.tCharacter = tLoad
		self.tCharacter.nVersion = self.nVersion
		self:AddCharacter()	
	end
end

function NeighborNotes:OnConfigure()
	NNConfigForm:Open()
end


------------------------------------------------------------------------------------------
-- Generic Event Handlers
------------------------------------------------------------------------------------------
function NeighborNotes:OpenNN(bShow)
	NNMainForm:Open(bShow)
end

function NeighborNotes:OnMessage(tChannel, tMsg, strSender)
	NNNetwork:OnMessage(tChannel, tMsg, strSender)
end

-- Registers Neighbor Notes with the Interface Menu
function NeighborNotes:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Neighbor Notes", { "ToggleAddon_NN", "", "IconSprites:Icon_Windows32_UI_CRB_InterfaceMenu_SupportTicket"})
	if self.bVisitor == nil then
		local addVisitor = Apollo.GetAddon("Visitor")
		self.bVisitor = false
		if addVisitor then
			self.bVisitor = true
		end
	end
	if self.aHousingTour == nil then
		self.aHousingTour = Apollo.GetAddon("HousingTour")
		self.bHousingTour = false
		if self.aHousingTour then
			self.bHousingTour = true
		end
	end
	self:SetInviteAddon()
	NNNetwork:Init()
	NNNetwork:CheckExistingInfo()
	for nNum, tCurrGuild in ipairs(GuildLib.GetGuilds()) do
		tCurrGuild:RequestMembers()
	end
	self:OnChangeZoneName( nil, GetCurrentZoneName())
	self:AutoRemove()
	self:RepairNotify()
	
end

function NeighborNotes:OnStartScan(strCommand, strPlayerInput)
	if not HousingLib.IsHousingWorld() then
		return
	end
	self.tLiveData.nScan = tonumber(strPlayerInput)	
	if not self.tLiveData.nScan or self.tLiveData.nScan < 1 then
		return
	end
	if not self.tLiveData.tSelectedNeighbor then
		if not self.tDisplayedList then
			self:RefreshList()
		end
		if not self.tDisplayedList then
			self.tLiveData.nScan = 0
			return
		end	
		self.tLiveData.tSelectedNeighbor = self.tDisplayedList[1]
		if not self.tLiveData.tSelectedNeighbor then
			self.tLiveData.nScan = 0
			return
		end
	end
	
	if self.tLiveData.nScan > #self.tDisplayedList then
		self.tLiveData.nScan = #self.tDisplayedList
	end
	self.tLiveData.nScan = self.tLiveData.nScan - 1
	self:VisitSelected()
end

function NeighborNotes:OnNNVisit(strCommand, strPlayerInput)
	local tTemp = nil
	if not string.IsEmpty(strPlayerInput) then
		tTemp = self:SelectNeighborByName(strPlayerInput)
		if not tTemp then
			tTemp = self:GetNeighborByName(strPlayerInput)
			if tTemp then
				HousingLib.VisitNeighborResidence(tTemp.nId)
			elseif self.aHousingTour then
				Apollo.ParseInput("/ht " .. strName)
			end
			return
		end
	end
	self:VisitSelected()
end

function NeighborNotes:OnPlayerCurrencyChanged()
	if HousingLib.IsOnMyResidence() then
		NNNetwork:GatherData()
	end
end

-- Places units that are fed to the client into a table/queue named self.tQueuedUnits
function NeighborNotes:OnUnitCreated(unit)
	if HousingLib.IsHousingWorld() == false then
		return
	end
	if unit == null or not unit:IsValid() or unit == GameLib.GetPlayerUnit() then
		return
	end
	table.insert(self.tLiveData.tQueuedUnits, unit)
end

-- Refreshes the List Container that contains the neighbor data
function NeighborNotes:RefreshList()
	if not NNMainForm:IsShown() and not NNSmallForm:IsShown() then
		return
	end	

	-- Updates the Neighbor Notes data with the current neighbor list
	local tNeighbors = HousingLib.GetNeighborList()
	if not tNeighbors then
		return
	end
		
	-- Check for bad records
	for nIndex, tNeighbor in ipairs(tNeighbors) do
		if tNeighbor.strCharacterName and not self.tRealm.tNeighborList[tNeighbor.strCharacterName] then
			self.tRealm.tNeighborList[tNeighbor.strCharacterName] = self:NewNeighborTable()
		end
	end
	tNeighbors = nil
	
	-- set up the neighbor lookup table
	-- local tSortData = HousingLib.GetNeighborList() or {}
	local tSortData = {}

	-- add search to the display list
	if self.tLiveData.tFilters.bShowSearch then
		local tRemoveList = {}
		for strName, tData in pairs(self.tLiveData.tSearch) do
			local bProcess = true
			if self.tCharacter.tFilters.bHarvestable and not self:IsHarvestableNode(tData.nNodeID) then
				bProcess = false
			end
			if self.tLiveData.tFilters.nPlug and not self:IsPlugPresent(tData.tPlugs, self.tLiveData.tFilters.nPlug) then
				bProcess = false
			end
			if self:IsMyNeighbor(strName) then
				bProcess = false
				table.insert(tRemoveList, strName)
				local tPlots = {}
				for nNum, nNodeID in ipairs(tData.tPlugs) do
					table.insert(tPlots, { ["nPlotID"] = nNodeID } )
				end
				if tData.fNodeExpiration then
					table.insert(tData.tPlugs, { ["nPlotID"] = tData.nNodeID, ["fHoursLeft"] = self:GetTimeRemaining(tData.fNodeExpiration) } )
				else
					table.insert(tData.tPlugs, { ["nPlotID"] = tData.nNodeID } )
				end
				self:UpdateNoteNeighbor(strName, tData.tPlots, tData.tSkills, tData.eShare, tData.ePrivacy, true, true)
			end
			if bProcess then
				tNData = {}
				tNData.bHasNN = true
				tNData.strNote = ""
				tNData.nNodeID = tData.nNodeID or 0
				tNData.fNodeExpiration = tData.fNodeExpiration				
				tNData.strCharacterName = strName
				tNData.strGroups = self:GetGroups(strName)
				tNData.tSkills = tData.tSkills
				tNData.eShare = tData.eShare
				tNData.ePrivacy = tData.ePrivacy
				tNData.ePermissionNeighbor = 998
				tNData.nHarvestedMine = tData.nHarvestedMine
				if tData.tPlugs then
					tNData.tPlugs = tData.tPlugs
					tNData.nPlugWeight = #tData.tPlugs
				else
					tNData.nPlugWeight = 0
				end
				tNData.fLastOnline = 0
				table.insert(tSortData, tNData)
			end
		end
		for nIndex, strName in ipairs(tRemoveList) do
			self.tLiveData.tSearch[strName] = nil
		end
	end

	for strName, tData in pairs(self.tRealm.tNeighborList) do
		local bProcess = false
		if self.tLiveData.tFilters.bShowNeighbors and self:IsMyNeighbor(strName) then
			bProcess = true
		end
		if self.tLiveData.tFilters.bShowNonNeighbors and not self:IsMyNeighbor(strName) then
			bProcess = true
		end
		if bProcess then
			if self.tLiveData.tFilters.bShowCircles and string.IsEmpty(self:GetGroups(strName)) then 
				bProcess = false
			end
			if self.tCharacter.tFilters.bHarvestable and not self:IsHarvestableNode(tData.nNodeID) then
				bProcess = false
			end
			if self.tLiveData.tFilters.nPlug and not self:IsPlugPresent(tData.tPlugs, self.tLiveData.tFilters.nPlug) then
				bProcess = false
			end
		end
		if bProcess then
			tNData = {}
			tNData.bHasNN = tData.bHasNN
			tNData.strNote = tData.strNote or  ""
			tNData.nNodeID = tData.nNodeID or 0
			tNData.fNodeExpiration = tData.fNodeExpiration
			tNData.strGroups = self:GetGroups(strName)
			tNData.strCharacterName = strName
			tNData.tSkills = tData.tSkills
			tNData.ePrivacy = tData.ePrivacy
			tNData.eShare = tData.eShare
			tNData.nHarvestedMine = tData.nHarvestedMine
			if tData.tPlugs then
				tNData.tPlugs = tData.tPlugs
				tNData.nPlugWeight = #tData.tPlugs
			else
				tNData.nPlugWeight = 0
			end
			local tTemp = self:GetNeighborByName(strName)
			if tTemp then
				tNData.fLastOnline = tTemp.fLastOnline or 1.0
				tNData.ePermissionNeighbor = tTemp.ePermissionNeighbor or HousingLib.NeighborPermissionLevel.Normal
				tNData.nId = tTemp.nId
			else
				tNData.fLastOnline = 1.0
				tNData.ePermissionNeighbor = 999
			end
			
			table.insert(tSortData, tNData)
		end
	end
	
	-- Do our sort
	if tSortData and #tSortData > 0 then
		if self.fnSort then
			table.sort(tSortData, self.fnSort)
		end
	end
	
	self.tDisplayedList = tSortData
	
	NNMainForm:Refresh()
	NNSmallForm:Refresh()
end



function NeighborNotes:OnFriendshipUpdate(nFriendID)
	local tFriend = FriendshipLib.GetById(nFriendId)
	if tFriend.bFriend == true and tFriend.strCharacterName and tFriend.strCharacterName ~= "" then
		if tFriend.fLastOnline == 0 then
			if self.tCharacter.tAutomation.bAIFriend and not self:IsMyNeighbor(tFriend.strCharacterName) then
				if #HousingLib.GetNeighborList() < (99 + #self.tRealm.tCharacterNames) then
					Print("Sending Neighbor Invite to " .. tFriend.strCharacterName)
					HousingLib.NeighborInviteByName(tFriend.strCharacterName)
				end
			end
		end
	end
end

function NeighborNotes:AutoRemove()
	if not self.tCharacter.tAutomation.bARemove then
		return
	end
	local tNeighbors = HousingLib.GetNeighborList()
	local strMessage = ""
	if tNeighbors ~= nil then
		for nIndex, tRemove in ipairs(tNeighbors) do
			if not self:IsMyCharacter(tRemove.strCharacterName) then
				if tRemove.fLastOnline > (self.tCharacter.tAutomation.nARemoveTime * 7) then
					if self.tCharacter.tAutomation.bARemoveGuild and self:IsInGuild(tRemove.strCharacterName) then
						--
					elseif self.tCharacter.tAutomation.bARemoveCircle and self:IsInCircle(tRemove.strCharacterName) then
						--
					else
						NNMainForm:MsgBox("Auto-Removed Neighbor: " .. tRemove.strCharacterName)
						HousingLib.NeighborEvict(tRemove.nId)
					end
				end
			end
		end
	end
end

function NeighborNotes:RepairNotify()
	local bNotify = false
	if not self.tCharacter.tAutomation.bANotifyHarvest and not self.tCharacter.tAutomation.bANotifyHarvestAlt then
		return
	end
	if not self.timerRepairNotify then
		self.timerRepairNotify = ApolloTimer.Create(1800, true, "RepairNotify", self)
	end
	for strName, bOol in pairs(self.tRealm.tCharacterNames) do
		local tNeighborInfo = self.tRealm.tNeighborList[strName]
		if tNeighborInfo ~= nil and tNeighborInfo.fNodeExpiration ~= nil then
			local nTimeLeft = self:GetTimeRemaining(tNeighborInfo.fNodeExpiration)
			if nTimeLeft > 0 then
			
			else
				if self.tCharacter.tAutomation.bANotifyHarvest and strName == GameLib.GetPlayerUnit():GetName() then
					NNMainForm:MsgBox(strName .. "'s " .. self.tLiveData.tPlugs[tNeighborInfo.nNodeID] .. " needs repairs")
				elseif self.tCharacter.tAutomation.bANotifyHarvestAlt and strName ~= GameLib.GetPlayerUnit():GetName() then
					NNMainForm:MsgBox(strName .. "'s " .. self.tLiveData.tPlugs[tNeighborInfo.nNodeID] .. " needs repairs")
				end
			end
		end
	end
end

function NeighborNotes:AutoRepair()
	if not self.tCharacter.tAutomation then
		return
	elseif not self.tCharacter.tAutomation.bARepairHarvest and not self.tCharacter.tAutomation.bARepairOther then
		return
	end
	local nPlotCount = HousingLib.GetPlotCount()
	local nIndex = 1
	local nMoney = GameLib.GetPlayerCurrency():GetAmount()

	while nIndex < nPlotCount do
		local tPlot = HousingLib.GetPlot(nIndex)
		if tPlot ~= nil and tPlot.fRemainingHours == 0 then
			local tPlugItem = HousingLib.GetPlugItem(tPlot.nPlugItemId)[1]
			local nCost = 0
			local bRepair = false
			if self.tCharacter.tAutomation.bARepairHarvest and tPlugItem.tFlags.bIsUniqueHarvest then
				bRepair = true
			elseif self.tCharacter.tAutomation.bARepairOther then
				bRepair = true
			end
			if bRepair then				
				if tPlugItem.tRepairRequirements[1].nRequiredCost > nMoney then
					NNMainForm:MsgBox("Not enough money to repair " .. tPlugItem.strName)
				else
					NNMainForm:MsgBox("Auto Repaired " .. tPlugItem.strName)
					nMoney = nMoney - tPlugItem.tRepairRequirements[1].nRequiredCost
					HousingLib.RepairPlot(nIndex)
				end
			end
		end
		nIndex = nIndex + 1
	end
	NNNetwork:GatherData()
end

function NeighborNotes:OnNeighborInviteRecieved(strName)
	function GetCompareDetails(tNeighborInfo)
		if not tNeighborInfo then
			return nil
		end
		if not self.tRealm.tNeighborList then
			return nil
		end
		if tNeighborInfo.nNodeID and tNeighborInfo.nNodeID > 0 then
			local tReturn = {}
			tReturn.eTier = tNeighborInfo.nNodeID % 10
			if tReturn.eTier > 4 then
				tReturn.eTier = 4
			end
			if tNeighborInfo.nNodeID < 61 then
				tReturn.eNodeType = CraftingLib.CodeEnumTradeskill["Relic_Hunter"] -- Relic_Hunter
			elseif tNeighborInfo.nNodeID < 71 then
				tReturn.eNodeType = CraftingLib.CodeEnumTradeskill["Survivalist"] -- Survivalist
			else
				tReturn.eNodeType = CraftingLib.CodeEnumTradeskill["Mining"] -- Mining
			end
			return tReturn
		else
			return nil
		end
	end
	function CanHarvestMine(tMyNode, tSkills)
		if self.tCharacter.tAutomation.bAAMyHarvest and tMyNode ~= nil then
			if tSkills and #tSkills > 0 then
				for nIndex, tSkill in ipairs(tSkills) do
					if tMyNode.eNodeType == tSkill.eId then
						if tMyNode.eTier < tSkill.eTier then
							return true
						end
					end
				end
			end
		end
		return false
	end
		
	local aNeighborList = Apollo.GetAddon("NeighborList")
	local tAutomation = self.tCharacter.tAutomation
	if not tAutomation.bAA then
		if aNeighborList then
			aNeighborList:OnNeighborInviteRecieved(strName)
			return
		end
	end
	local tPlayerInfo = self.tLiveData.tSearch[strName]
	local tPlayerNode = GetCompareDetails(tPlayerInfo)
	local tMyInfo = self.tRealm.tNeighborList[GameLib.GetPlayerUnit():GetName()]
	local tMyNode = GetCompareDetails(tMyInfo)

	local bAccept = false
	if tAutomation.bAACircle and self:IsInCircle(strName) then
		bAccept = true
	elseif tAutomation.bAAGuild and self:IsInGuild(strName) then
		bAccept = true
	elseif tAutomation.bAAReject then
		bAccept = false
	elseif not tAutomation.bAAHarvest and not tAutomation.bAAMyHarvest then -- this is actually the default
		bAccept = true
	elseif not tPlayerInfo then
		bAccept = false
	elseif not tMyInfo then
		bAccept = false
	elseif CanHarvestMine(tMyNode, tPlayerInfo.tSkills) then
		bAccept = true
	elseif tAutomation.bAAHarvest and tPlayerNode ~= nil then
		if tAutomation.eAAHarvestShare <= tPlayerInfo.eShare then
			for nIndex, tSkill in ipairs(CraftingLib.GetKnownTradeskills()) do
				local tradeskill = CraftingLib.GetTradeskillInfo(tSkill.eId)
				if tSkill.eId == tPlayerNode.eNodeType and tradeskill.bIsActive then
					if bAAHarvestSkill and (tradeskill.eTier-1) == tPlayerNode.eTier then
						bAccept = true
					elseif tradeskill.eTier > tPlayerNode.eTier then
						bAccept = true
					end
				end
			end
		end
	end
	if bAccept then
		Print("Neighbor Invite Accepted for " .. strName)
		HousingLib.NeighborInviteAccept()
	else
		Print("Neighbor Invite Rejected for " .. strName)
		HousingLib.NeighborInviteDecline()
	end
	self:RefreshList()
end

function NeighborNotes:InviteNeighbor()
	if #self.tLiveData.tInvites ~= 0 then
		HousingLib.NeighborInviteByName(self.tLiveData.tInvites[1])
		table.remove(self.tLiveData.tInvites, 1)
	else
		self.timerInvites:Stop()
	end
end

function NeighborNotes:SetInviteAddon()
	if self.tCharacter.tAutomation.bAA then
		local aNeighborList = Apollo.GetAddon("NeighborList")
		if aNeighborList ~= nil then
			Apollo.RemoveEventHandler("HousingNeighborInviteRecieved", aNeighborList)
		end
		Apollo.RegisterEventHandler("HousingNeighborInviteRecieved","OnNeighborInviteRecieved", self)
	else
		if aNeighborList ~= nil then
			Apollo.RegisterEventHandler("HousingNeighborInviteRecieved", 	"OnNeighborInviteRecieved", aNeighborList)
		end
		Apollo.RemoveEventHandler("HousingNeighborInviteReceived", self)
	end
end

-- Gets Guild and Circle data
function NeighborNotes:OnGuildRosterReceived(groupCurr, tGroupRoster)
	if table.IsEmpty(tGroupRoster) then
		return
	end
	local nCount = groupCurr:GetOnlineMemberCount()
	local tGroup = nil
	for nIndex, tTemp in ipairs(self.tLiveData.tGroups) do
		if tTemp.strName == groupCurr:GetName() then
			tGroup = tTemp
			break
		end
	end
	if not tGroup then
		tGroup = { strName = groupCurr:GetName(), nType = groupCurr:GetType() }
		table.insert(self.tLiveData.tGroups, tGroup)
		tGroup.nOnlineMembers = nCount
	end
	-- Build the roster
	local tRoster = {}
	local bCheckNew = false
	if self.tRealm.nCharacterCount == nil then
		self.tRealm.nCharacterCount = 1
	end
	if tGroup.nOnlineMembers < nCount and #HousingLib.GetNeighborList() < (99 + self.tRealm.nCharacterCount) then
		if tGroup.nType == self.eGuildType["Guild"] and self.tCharacter.tAutomation.bAIGuild then
			bCheckNew = true
		elseif tGroup.nType == self.eGuildType["Circle"] and self.tCharacter.tAutomation.bAICircle then
			bCheckNew = true
		end
	end
	if bCheckNew then
		for nIndex, tCharacter in ipairs(tGroupRoster) do
			local bOnline = false
			if tCharacter.fLastOnline == 0 then
				bOnline = true
			end
			tRoster[tCharacter.strName] = bOnline
			if not tGroup.tRoster[tCharacter.strName] and bOnline == true then
				if not self:IsMyNeighbor(tCharacter.strName) and not self:IsMyCharacter(tCharacter.strName) then
					Print("Sending Neighbor Invite to " .. tCharacter.strName)
					if not self.timerInvite then
						self.timerInvites = ApolloTimer.Create(10, true, "InviteNeighbor", self)
						self.timerInvites:Start()
					end
					table.insert(self.tLiveData.tInvites, tCharacter.strName)
				end
			end		
		end	
	else
		for nIndex, tCharacter in ipairs(tGroupRoster) do
			local bOnline = false
			if tCharacter.fLastOnline == 0 then
				bOnline = true
			end
			tRoster[tCharacter.strName] = bOnline
		end
	end
	tGroup.nOnlineMembers = nCount
	tGroup.tRoster = tRoster
end

function NeighborNotes:OnGuildMemberChange(tCurrGuild)
	tCurrGuild:RequestMembers()
end

-- Updates self.strCurrentZone and self.bNeighborZone with current zone info
function NeighborNotes:OnChangeZoneName(oVar, strNewZone)
	if string.IsEmpty(strNewZone) then
		return
	end
	local strNeighborName = nil
	local strLastNeighbor = nil
	local bHarvested = nil
	if not string.IsEmpty(self.tLiveData.strNeighborName) then
		strLastNeighbor = self.tLiveData.strNeighborName
		if self.tHarvestCache.nToMe ~= 0 or self.tHarvestCache.nToOwner ~= 0 then
			bHarvested = true
			self:UpdateNeighborHarvestData()
			self.tHarvestCache.nToMe = 0
			self.tHarvestCache.nToOwner = 0
		end
		self.tLiveData.strNeighborName = ""
	end
	
	if HousingLib.IsHousingWorld() and not HousingLib:IsWarplotResidence() then
		if not HousingLib.IsOnMyResidence() then
			strNeighborName = string.match(strNewZone, "%b[]")
			if string.IsEmpty(strNeighborName) then
				return
			end
			strNeighborName = strNeighborName:sub(2, -2)
		else
			NNNetwork:GatherData()
			self:AutoRepair()
		end
		if self.tRealm.bOpenMinimized then
			NNSmallForm:Open()
		end
	elseif self.tRealm.bCloseOnExit then
		NNMainForm:Close()
		NNSmallForm:Close()
	end

	if not string.IsEmpty(strNeighborName) then
		self.tLiveData.strNeighborName = strNeighborName
	end
	NNNetwork:SendZoneChange(strNeighborName, nil, nil, strLastNeighbor, bHarvested, nil, nil)
	
	if self:IsMyNeighbor(strNeighborName) then
		self:ProcessDetectedItems()
	end

	self.tHarvestCache.nToMe = 0
	self.tHarvestCache.nToOwner = 0
	
		
	if self.tLiveData.nScan ~= nil and self.tLiveData.nScan > 0 then
		self.tLiveData.nScan = self.tLiveData.nScan - 1
		self:VisitSelected()
	end
end

function NeighborNotes:OnSocialWindowToggle()
	if self.tCharacter.bOpenWithSocial then
		local wndSocial = Apollo.FindWindowByName("SocialPanelForm")
		--if wndSocial and wndSocial:IsShown() then
		if wndSocial then
			if wndSocial:FindChild("SplashNeighborsBtn"):IsChecked() then
				NNMainForm:Open(true)
			end
		end
	end
end

function NeighborNotes:OnSocialWindowClose()
	if self.tCharacter.bCloseWithSocial then
		NNMainForm:Close()
	end
end

function NeighborNotes:OnHarvest(tData)
	if tData and HousingLib.IsHousingWorld() then
		if not NeighborNotes:IsMyCharacter(NeighborNotes.tLiveData.strNeighborName) then
			local nCount = 0
			for nIndex, tItem in ipairs(tData) do
				nCount = nCount + tItem.nCount
			end
			self.tHarvestCache.strName = NeighborNotes.tLiveData.strNeighborName
			self.tHarvestCache.nToOwner = self.tHarvestCache.nToOwner + nCount
		end
	end
end

function NeighborNotes:OnLoot(uItem, nCount)
	if uItem and HousingLib.IsHousingWorld() then
		if not NeighborNotes:IsMyCharacter(NeighborNotes.tLiveData.strNeighborName) then
			self.tHarvestCache.nToMe = self.tHarvestCache.nToMe + nCount
		end
	end
end


function NeighborNotes:ProcessDetectedItems()
	local tGuild = GuildLib.GetGuilds()
	if #tGuild > 0 then
		for nIndex, guildCurr in ipairs(GuildLib.GetGuilds()) do
			guildCurr:RequestMembers()
		end
	end
	-- see if this zone should be scanned
	if HousingLib.IsHousingWorld() == false then
		self.tLiveData.tQueuedUnits = {}
		if self.tRealm.bCloseOnExit then
			NNMainForm:Close()
			NNSmallForm:Close()
		end
		return
	end
	
	if #self.tLiveData.tQueuedUnits == 0 then
		Print("No units found in this Neighborhood")
		return
	end
	
	local unitPlayerDisposition = GameLib.GetPlayerUnit()
	if unitPlayerDisposition == nil or not unitPlayerDisposition:IsValid() then
		self.tLiveData.tQueuedUnits = {}
		return
	end

	local tNewPlugs = {}
	local tFoundPlugs = {}
	local nTableCount = 0
	local nBestNode = 0	
	for nIndex, udUnit in pairs(self.tLiveData.tQueuedUnits) do
		local strName = udUnit:GetName()
		if not string.IsEmpty(strName) then
			local nID = self:GetPlugIDByUnitName(strName)
			if nID ~= nil then
				if udUnit:GetType() == "Harvest" then
					if nID ~= nil and nID > nBestNode then
						nBestNode = nID
					end
				else
					if tFoundPlugs[nID] == nil then
						if self.tLiveData.tPlugs[nID] == "Festival" then
							nTableCount = nTableCount + 1
						else
							tFoundPlugs[nID] = true
							table.insert(tNewPlugs, { ["nPlotID"] = nID } )
						end
					end
				end			
			end
		end
	end
	self.tOldUnits = self.tLiveData.tQueuedUnits
	self.tLiveData.tQueuedUnits = {}
	
	if nBestNode > 0 then
		table.insert(tNewPlugs, { ["nPlotID"] = nBestNode })
		local tNeighborInfo = self:GetNoteNeighborByName(self.tLiveData.strNeighborName)
		if tNeighborInfo and tNeighborInfo.bHasNN then
			if tNeighborInfo.fNodeExpiration and self:GetTimeRemaining(tNeighborInfo.fNodeExpiration) <= 0 then
				tNeighborInfo.fNodeExpiration = os.time() + 259200
			end
		end
	end
	
	if nTableCount == 1 then
		table.insert(tNewPlugs, { ["nPlotID"] = 11 } )
	elseif nTableCount == 3 then
		table.insert(tNewPlugs, { ["nPlotID"] = 12 } )
	elseif nTableCount == 4 then
		table.insert(tNewPlugs, { ["nPlotID"] = 13 } )
	elseif nTableCount == 5 then
		table.insert(tNewPlugs, { ["nPlotID"] = 14 } )
	elseif nTableCount == 10 then
		table.insert(tNewPlugs, { ["nPlotID"] = 15 } )
	elseif nTableCount == 6 then
		table.insert(tNewPlugs, { ["nPlotID"] = 16 } )
	end

	if self.tCharacter.bShowDataInChat == true then
		if #tNewPlugs > 0 then
			local strPlugNames = self.tLiveData.strNeighborName .. "'s Plugs:"
			for nNum, tPlug in ipairs(tNewPlugs) do
				if nNum > 1 then
					strPlugNames = strPlugNames .. ","
				end
				strPlugNames = strPlugNames .. " " .. self.tLiveData.tPlugs[tPlug.nPlotID]
			end
			Print(strPlugNames)
		end
	end
	self:UpdateNoteNeighbor(self.tLiveData.strNeighborName, tNewPlugs, nil, nil, nil, nil, nil)
end


---------------------------------------------------------------------------------------------------
-- tNeighborNotes datatable access functions
---------------------------------------------------------------------------------------------------
-- Returns the text note based on the character name
function NeighborNotes:GetCharacterNote(strCharacterName)
	if not string.IsEmpty(strCharacterName) then
		if self.tRealm.tNeighborList ~= nil then
			if self.tRealm.tNeighborList[strCharacterName] ~= nil and self.tRealm.tNeighborList[strCharacterName].strNote ~= nil then
				return self.tRealm.tNeighborList[strCharacterName].strNote
			end
		end
	end
	return ""
end

-- Sets the text note for the specified character
function NeighborNotes:SetCharacterNote(strCharacterName, strNote)
	if not string.IsEmpty(strCharacterName) then
		if self.tRealm.tNeighborList[strCharacterName] == nil then
			self.tRealm.tNeighborList[strCharacterName] = {}
		end
		self.tRealm.tNeighborList[strCharacterName].strNote = strNote
		self:RefreshList()
	end
end

function NeighborNotes:GetPlugIDByUnitName(strName)
	return self.tLiveData.tPlugUnits[strName]
end

function NeighborNotes:GetPlugIDByName(strName)
	return self.tLiveData.tPlugNames[strName]
end

function NeighborNotes:GetPlugItemInformation()
	self.tLiveData.tPlugItemInfo = {}
	local nIndex = 1
	local strSomething = ""
	while nIndex < 1000 do
		local tInfo = HousingLib.GetPlugItem(nIndex)[1]
		if tInfo and tInfo.nId and tInfo.strName and tInfo.eType then
			table.insert(self.tLiveData.tPlugItemInfo, tInfo.nId .. " , " .. tInfo.strName )
		end
		nIndex = nIndex + 1
	end
	self.strSomething = strSomething
end

-- Returns the neighbor info for strName from HousingLib.GetNeighborList()
function NeighborNotes:GetNeighborByName(strName)
	if not string.IsEmpty(strName) then
		tNeighborList = HousingLib.GetNeighborList()
		for nIndex, tNeighbor in ipairs(tNeighborList) do
			if tNeighbor.strCharacterName == strName then
				return tNeighbor
			end
		end	
	end
	-- no neighbor found
	return nil
end

function NeighborNotes:GetNoteNeighborByName(strName)
	return self.tRealm.tNeighborList[strName]
end

function NeighborNotes:GetGroups(strNeighborName)
	local strGroups = ""
	for nIndex, tGuild in ipairs(self.tLiveData.tGroups) do
		if tGuild.tRoster[strNeighborName] ~= nil then
			if strGroups ~= "" then
				strGroups = strGroups .. "\n\r"
			end
			if tGuild.nType == GuildLib.GuildType_Circle then
				strGroups = strGroups .. tGuild.strName .. "(Circle)"
			elseif tGuild.nType == GuildLib.GuildType_Guild then
				strGroups = strGroups .. tGuild.strName .. "(Guild)"
			end
		end
	end
	if strGroups ~= "" then
		return strGroups
	else
		return nil
	end
end

function NeighborNotes:GetPlugName(ePlugId)
	local strNamee = ""
	local tHousingItem = HousingLib.GetPlugItem(ePlugId)
	if tHousingItem then
		strName = tHousingItem.strName
	end
	return strName
end

-- returns a table with tradeskill names as the index.  Inactive skills are excluded
function NeighborNotes:GetActiveTradeskills()
	local tReturnList = {}
	local tKnownSkills = CraftingLib.GetKnownTradeskills()
	if tKnownSkills then
		for nIndex, tSkill in ipairs(tKnownSkills) do
			local tInfo = CraftingLib.GetTradeskillInfo(tSkill.eId)
			if tInfo and tInfo.bIsActive then
				if tSkill.eId == CraftingLib.CodeEnumTradeskill["Relic_Hunter"] then
					tReturnList["Relic"] = true
				elseif tSkill.eId == CraftingLib.CodeEnumTradeskill["Survivalist"] then
					tReturnList["Survival"] = true
				elseif tSkill.eId == CraftingLib.CodeEnumTradeskill["Mining"] then
					tReturnList["Mining"] = true
				end
			end
		end
	end
	return tReturnList
end

function NeighborNotes:GetNameTooltip(tNeighbor)
	local strNameTooltip = ""
	if tNeighbor.bHasNN then
			strNameTooltip = strNameTooltip .. "Has Neighbor Notes"
		if tNeighbor.nHarvestedMine ~= nil then
			strNameTooltip = strNameTooltip .. "\nHas harvested your nodes " .. tNeighbor.nHarvestedMine .. " time(s)."
		end
		if tNeighbor.tSkills ~= nil then
			if strNameTooltip ~= "" then
				strNameTooltip = strNameTooltip .. " \n"
			end
			strNameTooltip = strNameTooltip .. "Tradeskills: " 
			for nNum, tSkill in ipairs(tNeighbor.tSkills) do
				if tSkill.eId then
					strNameTooltip = strNameTooltip .. CraftingLib.GetTradeskillInfo(tSkill.eId).strName .. " Tier " .. tSkill.eTier .. " | "
				end
			end
		end
		if tNeighbor.ePrivacy ~= nil then
			if strNameTooltip ~= "" then
				strNameTooltip = strNameTooltip .. "\nPrivacy Level: "
			else
				strNameTooltip = strNameTooltip .. "Privacy Level: "
			end
			if tNeighbor.ePrivacy == HousingLib.ResidencePrivacyLevel["Public"] then
				strNameTooltip = strNameTooltip .. "Public"
			elseif tNeighbor.ePrivacy == HousingLib.ResidencePrivacyLevel["NeighborsOnly"] then
				strNameTooltip = strNameTooltip .. "Neighbors Only"
			elseif tNeighbor.ePrivacy == HousingLib.ResidencePrivacyLevel["RoommatesOnly"] then
				strNameTooltip = strNameTooltip .. "Roommates Only"
			elseif tNeighbor.ePrivacy == HousingLib.ResidencePrivacyLevel["Private"] then
				strNameTooltip = strNameTooltip .. "Private"
			end
		end
	end
	return strNameTooltip
end

function NeighborNotes:GetSelectedNeighbor()
	return self.tLiveData.tSelectedNeighbor
end

function NeighborNotes:GetTimeRemaining(fExpiration) -- in hours
	local fHoursLeft = (fExpiration - os.time())/3600
	if fHoursLeft < 0 then
		fHoursLeft = 0
	end
	return fHoursLeft
end

function NeighborNotes:IsHarvestableNode(nNodeID)
	local tActiveSkills = self:GetActiveTradeskills()
	if nNodeID == nil or nNodeID == 0 then
		return false
	elseif nNodeID < 061 and tActiveSkills["Relic"] then
		return true
	elseif nNodeID > 60 and nNodeID < 071 and tActiveSkills["Survival"] then
		return true
	elseif nNodeID > 70 and nNodeID < 081 and tActiveSkills["Mining"] then
		return true
	end
	return false
end

function NeighborNotes:IsPlugPresent(tPlugs, nPlugID)
	for nNum, nPlotID in ipairs(tPlugs) do
		if nPlotID == nPlugID then
			return true
		end
	end
	return false
end

function NeighborNotes:IsMyCharacter(strName)
	if self.tRealm.tCharacterNames ~= nil and self.tRealm.tCharacterNames[strName] ~= nil then
		return true
	end
	return false
end

function NeighborNotes:IsMyNeighbor(strName)
	local tNeighborList = HousingLib.GetNeighborList()
	for key, tCurrNeighbor in pairs(tNeighborList) do
		if tCurrNeighbor.strCharacterName == strName then
			return true
		end
	end
	return false
end

function NeighborNotes:IsInGuild(strName)
	for nIndex, tTemp in ipairs(self.tLiveData.tGroups) do
		if tTemp.nType == self.eGuildType["Guild"] then
			if tTemp.tRoster[strName] ~= nil then
				return true
			end
		end
	end
	return false
end

function NeighborNotes:IsInCircle(strName)
	for nIndex, tTemp in ipairs(self.tLiveData.tGroups) do
		if tTemp.nType == self.eGuildType["Circle"] then
			if tTemp.tRoster[strName] ~= nil then
				return true
			end
		end
	end
	return false
end

function NeighborNotes:NewNeighborTable()
	tNeighborInfo = { 
			["strNote"] = "",
			["nNodeID"] = 0,
			["tPlugs"]	= {},
			["tSkills"] = {},
			["tHarvestHistory"] = nil,
			["eShare"] = nil,
			["ePrivacy"] = nil,
	}
	return tNeighborInfo
end

-- This builds the tLiveData.tNodes, tNodeUnits, tPlugs, and tPlugUnits tables with localized names
function NeighborNotes:BuildTablesFromLocale(tLocale)
	self.tMissingPlugs = {}
	self.tMissingPlugUnits = {}
	-- Load Locale data for Plots
	self.tLiveData.tPlugs = {}
	self.tLiveData.tPlugUnits = {}
	self.tLiveData.tPlugNames = {}
	self.tLiveData.nPlugs = 0
	self.tLiveData.nPlugUnits = 0
	for nId, tData in pairs(self.tEnums.tPlugs) do
		local tPlugTemp = tLocale.tPlugs[tData.strName]
		if tPlugTemp then
			self.tLiveData.nPlugs = self.tLiveData.nPlugs + 1
			if tPlugTemp.strName and tPlugTemp.strName ~= "" then
				self.tLiveData.tPlugNames[tPlugTemp.strName] = nId
				self.tLiveData.tPlugs[nId] = tPlugTemp.strName
				if not tPlugTemp.tUnits or #tPlugTemp.tUnits == 0 then
					table.insert(self.tMissingPlugUnits, tData.strName)
				end	
				for nIndex, strLocalName in ipairs(tPlugTemp.tUnits) do
					self.tLiveData.tPlugUnits[strLocalName] = nId
					self.tLiveData.nPlugUnits = self.tLiveData.nPlugUnits + 1
				end
			else
				Print("Neighbor Notes Error: Please contact the author and let him know that " .. nId .. " is not working for your locale.")
			end
		else
			table.insert(self.tMissingPlugs, tData.strName)
		end
	end
end

-- Tries to determine the local and then calls BuildTablesFromLocale
function NeighborNotes:SetLocaleData()
	-- Figure out the locale
	local strCancel = Apollo.GetString(1)
	local strLocale = ""
	if strCancel == "Abbrechen" then -- German
		self:BuildTablesFromLocale(self.tLocale.de)
		self.strLocale = "German"
	elseif strCancel == "Annulaer" then -- French
		self:BuildTablesFromLocale(self.tLocale.fr)
		self.strLocale = "French"
	elseif strCancel == "something" then -- Korean
		self:BuildTablesFromLocale(self.tLocale.ko)
		self.strLocale = "Korean"
	else
		self:BuildTablesFromLocale(self.tLocale.en)
		self.strLocale = "English"
	end
end

function NeighborNotes:RemoveSelectedPlayer()
	local tCurr = self.tLiveData.tSelectedNeighbor
	if tCurr and tCurr.nId then
		HousingLib.NeighborEvict(tCurr.nId)
	end
end

function NeighborNotes:RemoveSelectedPlayerNote()
	local tCurr = self.tLiveData.tSelectedNeighbor
	if tCurr then 
		if self.tRealm.tNeighborList[tCurr.strCharacterName] then
			self.tRealm.tNeighborList[tCurr.strCharacterName] = nil
		end
		self:RefreshList()
	end
end

function NeighborNotes:VisitSelected()
	self.tQueuedUnits = {}
	tCurr = self.tLiveData.tSelectedNeighbor
	if not tCurr then
		if self.aHousingTour then
			Apollo.ParseInput("/ht")
		elseif self.bVisitor then
			Apollo.ParseInput("/visit")
		end
	else
		if self.tCharacter.bOnVisitSelectNext then
			self:SelectNextNeighbor()
		end
		if not tCurr.nId and self.aHousingTour then
			strName = ""
			if tCurr and tCurr.strCharacterName then
				strName = tCurr.strCharacterName
			end
			Apollo.ParseInput("/ht " .. strName)
		elseif tCurr.nId then
			HousingLib.VisitNeighborResidence(tCurr.nId)
		end
	end
end

function NeighborNotes:HelperConvertToTime(nDays)
	if nDays == 0 then
		return Apollo.GetString("Neighbors_Online")
	end
	if nDays == nil then
		return ""
	end

	local tTimeInfo = {["name"] = "", ["count"] = nil}

	if nDays >= 365 then -- Years
		tTimeInfo["name"] = Apollo.GetString("CRB_Year")
		tTimeInfo["count"] = math.floor(nDays / 365)
	elseif nDays >= 30 then -- Months
		tTimeInfo["name"] = Apollo.GetString("CRB_Month")
		tTimeInfo["count"] = math.floor(nDays / 30)
	elseif nDays >= 7 then
		tTimeInfo["name"] = Apollo.GetString("CRB_Week")
		tTimeInfo["count"] = math.floor(nDays / 7)
	elseif nDays >= 1 then -- Days
		tTimeInfo["name"] = Apollo.GetString("CRB_Day")
		tTimeInfo["count"] = math.floor(nDays)
	else
		local fHours = nDays * 24
		local nHoursRounded = math.floor(fHours)
		local nMin = math.floor(fHours*60)

		if nHoursRounded > 0 then
			tTimeInfo["name"] = Apollo.GetString("CRB_Hour")
			tTimeInfo["count"] = nHoursRounded
		elseif nMin > 0 then
			tTimeInfo["name"] = Apollo.GetString("CRB_Min")
			tTimeInfo["count"] = nMin
		else
			tTimeInfo["name"] = Apollo.GetString("CRB_Min")
			tTimeInfo["count"] = 1
		end
	end

	return String_GetWeaselString(Apollo.GetString("CRB_TimeOffline"), tTimeInfo)
end

function NeighborNotes:SelectNeighbor(strName)
	self.tLiveData.tSelectedNeighbor = nil
	if not string.IsEmpty(strName) then
		for nNum, tNeighbor in ipairs(self.tDisplayedList) do
			if tNeighbor.strCharacterName == strName then
				self.tLiveData.tSelectedNeighbor = tNeighbor
				break
			end
		end
	end
	NNMainForm:UpdateSelection()
	NNSmallForm:UpdateSelection()
end

function NeighborNotes:SelectNextNeighbor()
	if not self.tDisplayedList or #self.tDisplayedList == 0 then
		return
	end
	if self.tLiveData.tSelectedNeighbor ~= nil then
		bFound = false
		bChanged = false
		for nNum, tNeighbor in ipairs(self.tDisplayedList) do
			if bFound then
				bChanged = true
				self.tLiveData.tSelectedNeighbor = tNeighbor
				break
			elseif tNeighbor.strCharacterName == self.tLiveData.tSelectedNeighbor.strCharacterName then
				bFound = true
			end
		end
		if bChanged == false then
			self.tLiveData.tSelectedNeighbor = self.tDisplayedList[1]
		end
	end
	NNMainForm:UpdateSelection()
	NNSmallForm:UpdateSelection()
end

function NeighborNotes:SelectNeighborByName(strName)
	if not self.tDisplayedList or #self.tDisplayedList == 0 then
		self:RefreshList()
		if not self.tDisplayedList or #self.tDisplayedList == 0 then
			return
		end
	end
	if not string.IsEmpty(strName) then
		for nNum, tNeighbor in ipairs(self.tDisplayedList) do
			if string.lower(tNeighbor.strCharacterName) == string.lower(strName) then
				self.tLiveData.tSelectedNeighbor = tNeighbor
				NNMainForm:UpdateSelection()
				NNSmallForm:UpdateSelection()
				return self.tLiveData.tSelectedNeighbor
			end
		end
	end
end

function NeighborNotes:UpdateNoteNeighbor(strName, tPlugs, tSkills, eShare, ePrivacy, bHasNN, bSkipRefresh)
	function GetNodeType(nNum)
		if not nNum then
			return 0
		end
		return nNum - (nNum % 10)
	end
	function GetNodeTier(nNum)
		return nNum % 10
	end
	if string.IsEmpty(strName) then
		return
	end
	local tNeighborInfo = self.tRealm.tNeighborList[strName]
	if tNeighborInfo == nil then
		self.tRealm.tNeighborList[strName] = self:NewNeighborTable()
		tNeighborInfo = self.tRealm.tNeighborList[strName]
	end
	if bHasNN ~= nil then
		tNeighborInfo.bHasNN = bHasNN
	end
	if ePrivacy ~= nil then
		tNeighborInfo.ePrivacy = ePrivacy
	end
	if eShare ~= nil then
		tNeighborInfo.eShare = eShare
	end
	if not table.IsEmpty(tSkills) then
		tNeighborInfo.tSkills = tSkills
	end
	if not table.IsEmpty(tPlugs) then
		local tNewPlots = {}
		local bHasNode = false
		for nNum, tPlot in ipairs(tPlugs) do
			if tPlot.nPlotID > 50 and tPlot.nPlotID < 80 then -- this is a node
				bHasNode = true
				if GetNodeType(tNeighborInfo.nNodeID) ~= GetNodeType(tPlot.nPlotID) then
					tNeighborInfo.nNodeID = tPlot.nPlotID
				elseif GetNodeTier(tNeighborInfo.nNodeID) < GetNodeTier(tPlot.nPlotID) then
					tNeighborInfo.nNodeID = tPlot.nPlotID
				end
				if tPlot.fHoursLeft then
					if tPlot.fHoursLeft == 0 then
						tNeighborInfo.fNodeExpiration = os.time()
					else
						tNeighborInfo.fNodeExpiration = math.abs((tPlot.fHoursLeft*3600) + os.time())
					end
				end
			else
				table.insert(tNewPlots, tPlot.nPlotID)
			end
		end
		if tNeighborInfo.bHasNN and not bHasNode then
			tNeighborInfo.nNodeID = nil
			tNeighborInfo.fNodeExpiration = nil
		end
		table.sort(tNewPlots)
		tNeighborInfo.tPlugs = tNewPlots
	end
	if not bSkipRefresh then
		self:RefreshList()
	end
end

function NeighborNotes:UpdateSearch(strName, tPlugs, tSkills, eShare, ePrivacy)
	function GetNodeType(nNum)
		return nNum - (nNum % 10)
	end
	function GetNodeTier(nNum)
		return nNum % 10
	end
	if string.IsEmpty(strName) then
		return
	end
	if not tPlugs and not tSkills and not eShare and not ePrivacy then
		self.tLiveData.tSearch[strName] = nil
	else
		local tNeighborInfo = self.tLiveData.tSearch[strName]
		if tNeighborInfo == nil then
			self.tLiveData.tSearch[strName] = self:NewNeighborTable()
			tNeighborInfo = self.tLiveData.tSearch[strName]
		end
		tNeighborInfo.bUpdated = true
		tNeighborInfo.bHasNN = true
		if ePrivacy ~= nil then
			tNeighborInfo.ePrivacy = ePrivacy
		end
		if eShare ~= nil then
			tNeighborInfo.eShare = eShare
		end
		if not table.IsEmpty(tSkills) then
			tNeighborInfo.tSkills = tSkills
		end
		if not table.IsEmpty(tPlugs) then
			local tNewPlots = {}
			for nNum, tPlot in ipairs(tPlugs) do
				if tPlot.nPlotID > 50 and tPlot.nPlotID < 80 then -- this is a node
					if GetNodeType(tNeighborInfo.nNodeID) ~= GetNodeType(tPlot.nPlotID) then
						tNeighborInfo.nNodeID = tPlot.nPlotID
						tNeighborInfo.fNodeExpiration = (tPlot.fHoursLeft*3600) + os.time()
					elseif GetNodeTier(tNeighborInfo.nNodeID) < GetNodeTier(tPlot.nPlotID) then
						tNeighborInfo.nNodeID = tPlot.nPlotID
					end
				else
					table.insert(tNewPlots, tPlot.nPlotID)
				end
			end
			table.sort(tNewPlots)
			tNeighborInfo.tPlugs = tNewPlots
		end
	end
	if self.tLiveData.tFilters.bShowSearch then
		self:RefreshList()
	end
end

function NeighborNotes:UpdateSearchList()
	local tRemove = {}
	for strName, tData in pairs(self.tLiveData.tSearch) do
		if tData.bUpdated then
			tData.bUpdated = nil
		else
			table.insert(tRemove, strName)
		end
	end
	for nIndex, strName in ipairs(tRemove) do
		self.tLiveData.tSearch[strName] = nil
	end
	tRemove = {}
end

function NeighborNotes:RemoveFromSearch(strName)
	if string.IsEmpty(strName) then
		return
	end
	self.tLiveData[strName] = nil
end

function NeighborNotes:UpdateNeighborHarvestData()
	local tNeighbor = self.tRealm.tNeighborList[self.tLiveData.strNeighborName]
	if not tNeighbor then
		return
	end
	if not tNeighbor.tHarvestHistory then
		tNeighbor.tHarvestHistory = {}
	end
	local tHarvestHistory = tNeighbor.tHarvestHistory
	if #tHarvestHistory > 5 then
		table.remove(tHarvestHistory, 1)
	end
	table.insert(tHarvestHistory, {["nTotal"] = self.tHarvestCache.nToMe + self.tHarvestCache.nToOwner, ["nKept"] = self.tHarvestCache.nToMe})
	-- calculate new %
	if not tNeighbor.bHasNN then
		local nTotal = 0
		local nKept = 0
		for nIndex, tData in ipairs(tNeighbor.tHarvestHistory) do
			nTotal = nTotal + tData.nTotal
			nKept = nKept + tData.nKept
		end
		fPercent = nKept / nTotal
		if fPercent > 0.88 then 
			tNeighbor.eShare = NNNetwork.eShare["100"]
		elseif fPercent > 0.63 then
			tNeighbor.eShare = NNNetwork.eShare["75"]
		elseif fPercent > 0.38 then
			tNeighbor.eShare = NNNetwork.eShare["50"]
		elseif fPercent > 0.13 then
			tNeighbor.eShare = NNNetwork.eShare["25"]
		else
			tNeighbor.eShare = NNNetwork.eShare["0"]
		end
	end
end

function NeighborNotes:IncrementHarvest(strName)
	if string.IsEmpty(strName) then
		return
	end
	local tNeighborInfo = self.tRealm.tNeighborList[strName]
	if tNeighborInfo ~= nil then
		if tNeighborInfo.nHarvestedMine then
			tNeighborInfo.nHarvestedMine = tNeighborInfo.nHarvestedMine + 1
		else
			tNeighborInfo.nHarvestedMine = 1
		end
	end
end
	
function NeighborNotes:AddCharacter()
	local tUnit = GameLib.GetPlayerUnit()
	if tUnit == nil then
		return
	end
	local strName = tUnit:GetName()
	if string.IsEmpty(strName) then
		return
	end
	if self.tRealm.tCharacterNames == nil then
		self.tRealm.tCharacterNames = {}
	end
	self.tRealm.tCharacterNames[strName] = true
	local nCount = 0
	for strCharacter, bBool in pairs(self.tRealm.tCharacterNames) do
		nCount = nCount + 1
	end
	self.tRealm.nCharacterCount = nCount
end

-- Sets the sort order for the list
-- bOrder: if true then sort low to high, otherwise high to low
-- bNumeric: if true, treats the sort as numeric, otherwise as string
-- bDropEmpty: if true, drops empty records to the bottom of the table
function NeighborNotes:SetSort(eType, bOrder)
	-- If the string is blank, assign it "zzzzz" so that it sorts under non-blank strings 
	function SWeight(strText, bDrop)
		if bDrop then
			if strText == nil or strText == "" then
				return "zzzzz"
			end
		end
		return strText
	end
	-- if the number is nil, assign it 0 so that it doesn't break the sort
	function NWeight(nNumber, bDrop)
		if bDrop and nNumber == nil then
			return 0
		end
		return nNumber
	end

	if self.fnSort then
		self.fnLastSort = self.fnSort
	end
	
	local bNumeric = false
	local bDropEmpty = false
	local strField = ""
	
	if eType == self.tEnums.tSortType["Name"] then
		strField = "strCharacterName"
	elseif eType == self.tEnums.tSortType["Node"] then
		strField = "nNodeID"
		bNumeric = true
	elseif eType == self.tEnums.tSortType["Plug"] then
		strField = "nPlugWeight"
		bNumeric = true
	elseif eType == self.tEnums.tSortType["Note"] then
		strField = "strNote"
		bDropEmpty = true
	elseif eType == self.tEnums.tSortType["LastOnline"] then
		strField = "fLastOnline"
		bNumeric = true
		bOrder = not bOrder
	elseif eType == self.tEnums.tSortType["Permissions"] then
		strField = "ePermissionNeighbor"
		bNumeric = true
	end
	
	if strField == "strCharacterName" then
		if bOrder then
			self.fnSort = function(a,b) return (a[strField] < b[strField]) end
		else
			self.fnSort = function(a,b) return (a[strField] > b[strField]) end
		end
	elseif bNumeric then
		if bOrder then
			self.fnSort = function(a,b)
				if a[strField] == b[strField] then
					return (a["strCharacterName"] < b["strCharacterName"])
				else
					return (NWeight(a[strField], bDropEmpty) > NWeight(b[strField], bDropEmpty)) 
				end
			end
		else
			self.fnSort = function(a,b) 
				if a[strField] == b[strField] then
					return (a["strCharacterName"] < b["strCharacterName"])
				else
					return (NWeight(a[strField], bDropEmpty) < NWeight(b[strField], bDropEmpty))
				end
			end
		end
	else
		if bOrder then
			self.fnSort = function(a,b) 
				if a[strField] == b[strField] then
					return (a["strCharacterName"] < b["strCharacterName"])
				else
					return (SWeight(a[strField], bDropEmpty) < SWeight(b[strField], bDropEmpty))
				end
			end
		else
			self.fnSort = function(a,b) 
				if a[strField] == b[strField] then
					return (a["strCharacterName"] < b["strCharacterName"])
				else
					return (SWeight(a[strField], bDropEmpty) > SWeight(b[strField], bDropEmpty))
				end
			end
		end
	end
	self:RefreshList()
end

function NeighborNotes:Reset()
	self.tCharacter = self:CharacterDefaults()
	self.tRealm = self:RealmDefaults()
	self.tAccount = {}
	RequestReloadUI(true)
end

function NeighborNotes:CharacterDefaults()
	return {
		["nVersion"] = self.nVersion,
		["bOpenWithSocial"] 	= true,
		["bCloseWithSocial"] 	= true,
		["bOnVisitSelectNext"] 	= true,
		["bShowChatVisits"] 	= true,
		["bShowChatHarvests"] 	= true,
		["bShareData"] 			= true,
		["bShareHarvest"] 		= true,
		
		["tFilters"] = { 	
			["bShowNotes"] = true, 	
		},
	
		["tBroadcastData"] = { },
		
		["tAutomation"] = {
			["bAA"] = 				true,
			["bAAGuild"] = 			true,
			["bAACircle"] = 		true,
			["bARepairHarvest"] = 	true,
			["bANotifyHarvest"] = 	true,
			["bANotifyHarvestAlt"] = true,
		}
	}
end

function NeighborNotes:RealmDefaults()
	return {
		["nVersion"] = self.nVersion,
		["bCloseOnExit"] = true,
		["bOpenMinimized"] = true,
		["tNeighborList"] = {},
		["tCharacterNames"] = {},
	}
end

-- Initialize the Addon
local NeighborNotesInst = NeighborNotes:new()
NeighborNotes:Init()

