NNNetwork = {}

NNNetwork.eMsgType = { 
	["Residence"] = 1, 
	["ZoneChange"] = 2, 
	["Disconnect"] = 3,
}
NNNetwork.eShare = {
	["0"] = 0, -- Owner keeps all
	["25"] = 1, 
	["50"] = 2,
	["75"] = 3,
	["100"] = 4	-- harvester keeps all
}

NNNetwork.tNonNeighbors = {}

NNNetwork.tMessages = {}
NNNetwork.tMessages.tBroadcastData = nil
NNNetwork.tMessages.tBroadcastDefaults = {
	["eMsgType"] = NNNetwork.eMsgType["Residence"],
	["ePrivacy"] = 0,
	["eShare"] = 0,
	["bShowInSearch"] = true,
	["tSkills"] = {},
	["tPlots"] = {},
	["tUnknownPlots"] = {},
}

NNNetwork.tMessages.tZoneChangeData = {
	["nVer"] = 10300,
	["eMsgType"] = NNNetwork.eMsgType["ZoneChange"],
	["strOwner"] = nil,
	["nTotalVisits"] = nil,
	["nWeeklyVisits"] = nil,
	-- this data applies to the last person visited
	["strLastOwner"] = nil,
	["bHarvested"] = nil,
	["nTotalHarvest"] = nil,
	["nWeeklyHarvest"] = nil,
}

function NNNetwork:Init()
	if not NeighborNotes.tCharacter.bDisableICC then
		self.comChan = ICCommLib.JoinChannel("NeighborShare", "OnMessage", NeighborNotes)
	else
		NNMainForm:MsgBox("ICC is Disabled")
		self.comChan = nil
	end
	
	if not self.comChan then
		self.bNetworkFailed = true
		Apollo.AddAddonErrorText("Could not join channel")
		return
	end
	self.timerBroadcast = ApolloTimer.Create(300, true, "OnTimer", self)
end

function NNNetwork:OnMessage(tChannel, tMsg, strSender)
	if not tMsg then 
		return
	end
	if not tMsg.eMsgType then
		return
	end
	if string.IsEmpty(strSender) then
		return
	end

	local tNeighborInfo = NeighborNotes.tRealm.tNeighborList[strSender]
	if tNeighborInfo == nil then
		if tMsg.eMsgType == self.eMsgType["Residence"] and not tMsg.bHideFromSearch then
			NeighborNotes:UpdateSearch(strSender, tMsg.tPlots, tMsg.tSkills, tMsg.eShare, tMsg.ePrivacy)
		end
		return
	end
	
	if not NeighborNotes:IsMyNeighbor(strSender) then
		if tMsg.eMsgType == self.eMsgType["Residence"] and not tMsg.bHideFromSearch then
			NeighborNotes:UpdateSearch(strSender, tMsg.tPlots, tMsg.tSkills, tMsg.eShare, tMsg.ePrivacy)
		end
	end
	
	local tTTime = GameLib.GetLocalTime()
	local bHunt = false
	if tMsg.eMsgType == self.eMsgType["ZoneChange"] then
		self:ProcessZoneChange(strSender, tMsg)
	elseif tMsg.eMsgType == self.eMsgType["Residence"] then
		if bHunt then
			self:PrintResidencePacket(strSender, tMsg)
		end
		NeighborNotes:UpdateNoteNeighbor(strSender, tMsg.tPlots, tMsg.tSkills, tMsg.eShare, tMsg.ePrivacy, true, false)
	else
		if tMsg.eMsgType then
			Print("Got Unknown Message from " .. strSender .. " MessageID: " .. tostring(tMsg.eMsgType))
		else
			Print("Got Unknown Message from " .. strSender .. " with no message ID")
		end
	end
end

function NNNetwork:ProcessZoneChange(strSender, tMsg)
	if string.IsEmpty(tMsg.strOwner) and string.IsEmpty(tMsg.strLastOwner) then
		return
	end
	if not tMsg.nVer then
		if string.IsEmpty(tMsg.strOwner) then
			return
		end
		tMsg.strLastOwner = tMsg.strOwner
		tMsg.strOwner = nil
		tMsg.bHarvested = true
		tMsg.nVer = NNNetwork.tMessages.tZoneChangeData.nVer
	end

	if not string.IsEmpty(tMsg.strOwner) then
		if NeighborNotes:IsMyCharacter(tMsg.strOwner) then
			if NeighborNotes.tCharacter.bShowChatVisits then
				Print(strSender .. " is visiting " .. tMsg.strOwner)
			end
		end
	end
	if not string.IsEmpty(tMsg.strLastOwner) then
		if NeighborNotes:IsMyCharacter(tMsg.strLastOwner) then
			local strMessage = strSender .. " has left " .. tMsg.strLastOwner .. "'s house"
			if tMsg.bHarvested then
				NeighborNotes:IncrementHarvest(strSender)
				NeighborNotes:RefreshList()
				strMessage = strMessage .. " and did some harvesting while there."
			end
			if NeighborNotes.tCharacter.bShowChatVisits or NeighborNotes.tCharacter.bShowChatHarvests then
				Print(strMessage)
			end
		end
	end
end

function NNNetwork:CheckExistingInfo()
	if not self.comChan then
		return
	end
	if NeighborNotes.tCharacter.tBroadcastData then
		self.tMessages.tBroadcastData = NeighborNotes.tCharacter.tBroadcastData
		self:OnTimer()
		self.timerBroadcast:Start()
	end
end

function NNNetwork:OnTimer()
	if NNNetwork.tMessages.tBroadcastData then
		NNNetwork:SendMessage(NNNetwork.tMessages.tBroadcastData)
	else
		self.timerBroadcast:Stop()
	end
	NeighborNotes:UpdateSearchList()
end

function NNNetwork:SendMessage(tData)
	if not self.comChan then
		return
	end
	if tData then
		self.comChan:SendMessage(tData)
	end
end

function NNNetwork:SendZoneChange(strNeighborName, nTotalVisits, nWeeklyVisits, strLastNeighbor, bHarvested, nTotalHarvest, nWeeklyHarvest)
	self.tMessages.tZoneChangeData["strOwner"] = strNeighborName
	self.tMessages.tZoneChangeData["nTotalVisits"] = nTotalVisits
	self.tMessages.tZoneChangeData["nWeeklyVisits"] = nWeeklyVisits
	self.tMessages.tZoneChangeData["strLastNeighbor"] = strLastNeighbor
	self.tMessages.tZoneChangeData["bHarvested"] = bHarvested
	self.tMessages.tZoneChangeData["nTotalHarvest"] = nTotalHarvest
	self.tMessages.tZoneChangeData["nWeeklyHarvest"] = nWeeklyHarvest
	self:SendMessage(NNNetwork.tMessages.tZoneChangeData)
end

function NNNetwork:SendHarvestMessage()
	if NeighborNotes.tCharacter.bShareData and NeighborNotes.tCharacter.bShareHarvest then
		self.tMessages.tHarvestData["strOwner"] = self.tHarvestCache.strName
		self:SendMessage(NNNetwork.tMessages.tHarvestData)
		self.tHarvestCache.strName = nil
	end
end

function NNNetwork:GatherData()
	if not NeighborNotes.tCharacter.bShareData then
		return
	end
	local tUnknownPlots = {}
	local tPlots = {}
	local nNum = 1
	self.tMessages.tBroadcastData = {}
	self.tMessages.tBroadcastData["eMsgType"] = self.eMsgType["Residence"]
	self.tMessages.tBroadcastData["ePrivacy"] = HousingLib.GetResidencePrivacyLevel()
	self.tMessages.tBroadcastData["eShare"] = HousingLib:GetNeighborHarvestSplit()
	self.tMessages.tBroadcastData["tSkills"] = self:GetKnownCraftingInfo()
	self.tMessages.tBroadcastData["bHideFromSearch"] = NeighborNotes.tCharacter.bHideFromSearch	
	self.tMessages.tBroadcastData["tPlots"] = {}
	self.tMessages.tBroadcastData["tUnknownPlots"] = {}
	while true do
		local tPlot = HousingLib.GetPlot(nNum)
		if tPlot then
			if tPlot.eUpkeepType == 1 then
				nPlotId = NeighborNotes:GetPlugIDByName(tPlot.strName)
				if nPlotId then
					table.insert(self.tMessages.tBroadcastData.tPlots, 
								{ ["nPlotID"] = nPlotId, ["fHoursLeft"] = tPlot.fRemainingHours } )
				else
					table.insert(self.tMessages.tBroadcastData.tUnknownPlots, 
								{ ["strName"] = tPlot.strName, ["nItemID"] = tPlot.nPlugItemId, ["fHoursLeft"] = tPlot.fRemainingHours } )
				end
			end
			nNum = nNum + 1
		else
			break;
		end
	end
	NeighborNotes.tCharacter.tBroadcastData = self.tMessages.tBroadcastData
	NeighborNotes:UpdateNoteNeighbor(GameLib.GetPlayerUnit():GetName(), self.tMessages.tBroadcastData.tPlots, self.tMessages.tBroadcastData.tSkills, self.tMessages.tBroadcastData.eShare, self.tMessages.tBroadcastData.ePrivacy, true, false)
	if not self.bNetworkFailed and self.timerBroadcast then
		self:OnTimer()
		self.timerBroadcast:Start()
	end
end

-- returns a table with known crafting info
function NNNetwork:GetKnownCraftingInfo()
	local tSkills = CraftingLib.GetKnownTradeskills()
	local tReturn = {}
	for nNum, tSkill in ipairs(tSkills) do
		local tSkillInfo = CraftingLib.GetTradeskillInfo(tSkill.eId)
		if tSkillInfo.bIsActive and tSkillInfo.bIsHarvesting then
			if tSkill.eId ~= CraftingLib.CodeEnumTradeskill["Farmer"] then
				table.insert(tReturn, { ["eId"] = tSkill.eId, ["eTier"] = tSkillInfo.eTier })
			end
		end
	end
	return tReturn
end

function NNNetwork:PrintResidencePacket(strSender, tMsg)
	local strTest = "Name: " .. strSender .. " | Privacy = "
	if tMsg.ePrivacy == 0 then
		strTest = strTest .. "Public "
	else
		strTest = strTest .. "Private "
	end
	strTest = strTest .. " | Shared % = "
	if tMsg.eShare == 4 then
		strTest = strTest .. "100 "
	elseif tMsg.eShare == 3 then
		strTest = strTest .. "75 "
	elseif tMsg.eShare == 2 then
		strTest = strTest .. "50 "
	elseif tMsg.eShare == 1 then
		strTest = strTest .. "25 "
	else
		strTest = strTest .. "0 "
	end
	if tMsg.tSkills then
		strTest = strTest .. "\nTradeskills = "
		for nNum, tSkill in ipairs(tMsg.tSkills) do
			if tSkill.eId then
				strTest = strTest .. CraftingLib.GetTradeskillInfo(tSkill.eId).strName .. " tier " .. tSkill.eTier .. ", "
			end
		end
	end
	if tMsg.tPlots then
		strTest = strTest .. "\nPlots:"
		for nNum, tPlot in ipairs(tMsg.tPlots) do
			strTest = strTest .. " \n     " .. NeighborNotes.tLiveData.tPlugs[tPlot.nPlotID] .. " Hours Left: " .. tostring(tPlot.fHoursLeft)
		end 
	end
	if tMsg.tUnknownPlots then
		strTest = strTest .. "\nUnknown Plots:"
		for nNum, tPlot in ipairs(tMsg.tUnknownPlots) do
			strTest = strTest .. " \n     " .. tPlot.strName .. " eId: " .. tPlot.nItemID .. " Hours Left: " .. tostring(tPlot.fHoursLeft)
		end
	end
	Print(strTest)
end

