NNSmallForm = {}

NNSmallForm.tPixies = {}

NNSmallForm.tPixies.tHarvest = {
	strText = "",
	strFont = "CRB_Interface9",
	strSprite = "NNSprites:NN_Harvest_100",
	bLine = false,
	crText = "ffffffff",
	cr = "ffff0000",
	loc = { fPoints = { 0, 0, 1, 1 }, nOffsets = { 0, 0, 0, 0 } },
	flagsText = { DT_BOTTOM = false, DT_RIGHT = false, DT_CENTER = false, DT_VCENTER = false }
}


NNSmallForm.tPixies.tNodeTier = {
	strText = "",
	strFont = "CRB_Pixel_O",
	strSprite = "",	
	bLine = false,
	crText = "ffffff00",
	loc = { fPoints = { 0, 0, 1, 1 }, nOffsets = { 0, 0, 0, 0 } },
	flagsText = { DT_BOTTOM = true, DT_RIGHT = true, DT_CENTER = false, DT_VCENTER = false } 
}

function NNSmallForm:Load()
	self.xmlDoc = XmlDoc.CreateFromFile("SmallForm.xml")
	if not self.xmlDoc then
		Apollo.AddAddonErrorText("Could not load SmallForm")
	end
end

function NNSmallForm:Init()
	if not self.wndMain or not self.wndMain:IsValid() then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "SmallForm", nil, self)
		if not self.wndMain then
			Apollo.AddAddonErrorText(self, "Could not load the NeighborNotes Small window.")
			return
		end
		Apollo.LoadSprites("NNSprites.xml")
		self.wndListContainer = self.wndMain:FindChild("ListContainer")
		Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "NeighborNotesSmall"})
	end
end

function NNSmallForm:Open()
	if not self.wndMain then
		self:Init()
		NeighborNotes:RefreshList()		
	end
	self.wndMain:Invoke()
	self.wndMain:ToFront()
end

function NNSmallForm:Close()
	if self.wndMain and self.wndMain:IsShown() then
		self.wndMain:Close()
	end
end

function NNSmallForm:Refresh()
	if not self.wndMain or not self.wndMain:IsShown() then
		return
	end

	self.wndListContainer:DestroyChildren()
	self.wndSelectedListItem = nil

	if not NeighborNotes.tDisplayedList then
		return
	end
	-- Populate the List Container
	for key, tCurrNeighbor in pairs(NeighborNotes.tDisplayedList) do
		local wndListItem = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndListContainer, self)
		self:UpdateListItem(wndListItem, tCurrNeighbor)
	end
	self.wndListContainer:ArrangeChildrenVert()
	self:UpdateSelection()
	self:UpdateControls()
end

function NNSmallForm:UpdateListItem(wndListItem, tNeighborInfo)
	wndListItem:SetData(tNeighborInfo)
	local strColorToUse = NeighborNotes.tColors.crOffline
	if tNeighborInfo.fLastOnline == 0 then -- online / check for strWorldZone
		strColorToUse = NeighborNotes.tColors.crOnline
	end
	local wndName = wndListItem:FindChild("Name")
	-- Set the name
	wndName:SetText(tNeighborInfo.strCharacterName)
	wndName:SetTextColor(strColorToUse)
	wndName:SetTooltip(NeighborNotes:GetNameTooltip(tNeighborInfo))
	-- Set the node
	if tNeighborInfo.nNodeID and tNeighborInfo.nNodeID ~= 0 then
		local wndNode = wndListItem:FindChild("Node")
		if tNeighborInfo.nNodeID > 50 and tNeighborInfo.nNodeID < 61 then
			wndNode:SetSprite(NeighborNotes.tIcons.strRelic)
		elseif tNeighborInfo.nNodeID > 60 and tNeighborInfo.nNodeID < 71 then
			wndNode:SetSprite(NeighborNotes.tIcons.strTree)
		elseif tNeighborInfo.nNodeID > 70 and tNeighborInfo.nNodeID < 81 then
			wndNode:SetSprite(NeighborNotes.tIcons.strMining)
		end
		local nNodeTier = tNeighborInfo.nNodeID % 10
		local strNodeTier = nil
		if nNodeTier > 4 then
			strNodeTier = "E"
		else
			strNodeTier = tostring(nNodeTier)
		end
		self.tPixies.tNodeTier.strText = strNodeTier
		wndNode:AddPixie(self.tPixies.tNodeTier)
		local strNodeTooltip = NeighborNotes.tLiveData.tPlugs[tNeighborInfo.nNodeID]
		if tNeighborInfo.eShare then
			if tNeighborInfo.eShare == 4 then
				self.tPixies.tHarvest.strSprite = "NNSprites:NN_Harvest_100"
				strNodeTooltip = strNodeTooltip .. "\n Get 100% of harvested goods"
			elseif tNeighborInfo.eShare == 3 then
				self.tPixies.tHarvest.strSprite = "NNSprites:NN_Harvest_75"
				strNodeTooltip = strNodeTooltip .. "\n Get 75% of harvested goods"
			elseif tNeighborInfo.eShare == 2 then
				self.tPixies.tHarvest.strSprite = "NNSprites:NN_Harvest_50"
				strNodeTooltip = strNodeTooltip .. "\n Get 50% of harvested goods"
			elseif tNeighborInfo.eShare == 1 then
				self.tPixies.tHarvest.strSprite = "NNSprites:NN_Harvest_25"
				strNodeTooltip = strNodeTooltip .. "\n Get 25% of harvested goods"
			else
				self.tPixies.tHarvest.strSprite = "NNSprites:NN_Harvest_0"
				strNodeTooltip = strNodeTooltip .. "\n Get 0% of harvested goods"
			end
			wndNode:AddPixie(self.tPixies.tHarvest)
		end
		if tNeighborInfo.fNodeExpiration ~= nil then
			local nTimeLeft = (tNeighborInfo.fNodeExpiration - os.time()) / 3600
			if nTimeLeft > 0 then
				strNodeTooltip = strNodeTooltip .. "\nExpires in " .. nTimeLeft .. " hours"
			else
				strNodeTooltip = strNodeTooltip .. "\nNeeds Repairs"
				self.tPixies.tHarvest.strSprite = "CRB_CharacterCreateSprites:sprCharC_ClassFooterIconDisabled"
				wndNode:AddPixie(self.tPixies.tHarvest)
			end
		end
		wndNode:SetTooltip(strNodeTooltip)
	end
	-- Set the plugs
	if tNeighborInfo.tPlugs ~= nil then
		for nIndex, nPlugID in ipairs(tNeighborInfo.tPlugs) do
			if nPlugID ~= nil and nPlugID ~= 0 then
				local wndPlug = wndListItem:FindChild("Plug" .. nIndex)
				if not wndPlug then -- There are currently 7 plug spots, any more than 7 will not be shown
					break
				end
				if NeighborNotes.tEnums.tPlugs[nPlugID].strIcon == nil then
					if nPlugID < 101 then
						wndPlug:SetSprite("IconSprites:Icon_CraftingUI_Item_Crafting_PowerCore_Blue")
					elseif nPlugID < 201 then
						wndPlug:SetSprite(NeighborNotes.tEnums.tPlugType["Biome"].strIcon)
					elseif nPlugID < 401 then
						wndPlug:SetSprite(NeighborNotes.tEnums.tPlugType["Challenge"].strIcon)
					elseif nPlugID < 601 then
						wndPlug:SetSprite(NeighborNotes.tEnums.tPlugType["Expedition"].strIcon)
					elseif nPlugID < 801 then
						wndPlug:SetSprite(NeighborNotes.tEnums.tPlugType["Raid"].strIcon)
					elseif nPlugID < 1001 then
						wndPlug:SetSprite(NeighborNotes.tEnums.tPlugType["PublicEvent"].strIcon)
					end
				else
					wndPlug:SetSprite(NeighborNotes.tEnums.tPlugs[nPlugID].strIcon)
				end
				wndPlug:SetTooltip(NeighborNotes.tLiveData.tPlugs[nPlugID])
				-- Update Pixies for Festival Plugs
				if wndPlug:GetPixieInfo(1) then
					wndPlug:DestroyPixie(1)
				end
				if nPlugID > 010 and nPlugID < 017 then
					if nPlugID == 011 then
						self.tPixies.tNodeTier["strText"] = "1"
					elseif nPlugID == 012 then
						self.tPixies.tNodeTier["strText"] = "2"
					elseif nPlugID == 013 then
						self.tPixies.tNodeTier["strText"] = "3"
					elseif nPlugID == 014 then
						self.tPixies.tNodeTier["strText"] = "4"
					elseif nPlugID == 015 then
						self.tPixies.tNodeTier["strText"] = "5"
					elseif nPlugID == 016 then
						self.tPixies.tNodeTier["strText"] = "6"
					end
					wndPlug:AddPixie(self.tPixies.tNodeTier)
				end
			end
		end
	end
end

function NNSmallForm:UpdateControls()
	if not self.wndMain or not self.wndMain:IsShown() then
		return
	end
	local bHousing = HousingLib.IsHousingWorld()
	self.wndMain:FindChild("TeleportHomeBtn"):Enable(bHousing)
	self.wndMain:FindChild("VisitBtn"):Enable(bHousing)
end
	
function NNSmallForm:UpdateSelection()
	if not self.wndMain or not self.wndMain:IsValid() then
		return
	end
	if self.wndSelectedListItem then
		self.wndSelectedListItem:FindChild("SelectionBtn"):SetCheck(false)
		self.wndSelectedListItem = nil
	end
	if NeighborNotes.tLiveData.tSelectedNeighbor then
		local nPos = 1
		for nNum, wndListItem in ipairs(self.wndListContainer:GetChildren()) do
			local tCurr = wndListItem:GetData()	
			if tCurr.strCharacterName == NeighborNotes.tLiveData.tSelectedNeighbor.strCharacterName then
				wndListItem:FindChild("SelectionBtn"):SetCheck(true)
				self.wndSelectedListItem = wndListItem
				local nHeight = self.wndListContainer:GetHeight()
				nHeight = nHeight - nHeight%40
				nHeight = nHeight/40
				nHeight = nHeight - nHeight%2
				nHeight = nHeight/2
				self.wndListContainer:SetVScrollPos(nPos - nHeight*40)
				return
			end
			nPos = nPos + 40
		end
	end
end

function NNSmallForm:IsShown()
	if self.wndMain and self.wndMain:IsShown() then
		return true
	end
	return false
end

-- Events

function NNSmallForm:OnSelectionCheck( wndHandler, wndControl, eMouseButton )
	local tCurrNeighbor = wndControl:GetParent():GetData()
	if tCurrNeighbor == nil then
		return false
	end
	if self.wndSelectedListItem then
		self.wndSelectedListItem:FindChild("SelectionBtn"):SetCheck(false)
	end
	self.wndSelectedListItem = wndControl:GetParent()
	wndControl:SetCheck(true)
	NeighborNotes:SelectNeighbor(tCurrNeighbor.strCharacterName)
	self:UpdateControls()
end

function NNSmallForm:OnSelectionUncheck( wndHandler, wndControl, eMouseButton )
	if self.wndSelectedListItem then
		self.wndSelectedListItem:SetCheck(false)
	end
	NeighborNotes:SelectNeighbor()
end

function NNSmallForm:OnMaximizeBtn( wndHandler, wndControl, eMouseButton )
	NNMainForm:Open()
	self:Close()
end

function NNSmallForm:OnTeleportHomeBtn()
	HousingLib.RequestTakeMeHome()
end

function NNSmallForm:OnVisitBtn( wndHandler, wndControl, eMouseButton)
	NeighborNotes:VisitSelected()
end

function NNSmallForm:OnAddBtn( wndHandler, wndControl, eMouseButton)
	local wndAdd = wndControl:FindChild("AddWindow")
	if wndAdd:IsVisible() then
		wndAdd:Show(false)
	else
		local wndEditBox = wndAdd:FindChild("AddMemberEditBox")
		local tCurr = NeighborNotes.tLiveData.tSelectedNeighbor
		wndEditBox:SetText("")
		if tCurr and not tCurr.nId then
			wndEditBox:SetText(tCurr.strCharacterName or "")
		end
		wndAdd:FindChild("AddMemberEditBox"):SetFocus()
		wndAdd:Show(true)
		wndAdd:ToFront()
	end
end

function NNSmallForm:OnAddMemberYesClick( wndHandler, wndControl )
	local wndParent = wndControl:GetParent()
	local strName = wndParent:FindChild("AddMemberEditBox"):GetText()
	if strName ~= nil and strName ~= "" then
		HousingLib.NeighborInviteByName(strName)
	end
	wndControl:GetParent():Show(false)
end

function NNSmallForm:OnSubWindowClosed( wndHandler, wndControl )
	wndControl:GetParent():SetCheck(false)
end

function NNSmallForm:OnCloseBtn( wndHandler, wndControl, eMouseButton )
	self:Close()
end